/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file start.c
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun. 25, 2013
*
* @brief provide high-level startup routines for KExx.
*
*******************************************************************************/

#include "start.h"
#include "common.h"
#include "wdog.h"
#include "sysinit.h"

void start(void);
void SystemInit( void );
/********************************************************************/
/*!
 * \brief   Kinetis Start
 * \return  None
 *
 * This function calls all of the needed starup routines and then 
 * branches to the main process.
 */
void start(void)
{ 
#if  !defined(WDOG_ENABLED)
	/* Disable the watchdog timer */
    WDOG_Disable();
#endif

#ifndef __GNUC__ 
#ifndef KEIL
	/* Copy any vector or data sections that need to be in RAM */
	common_startup();
#endif
#endif

#if !defined(BYPASS_SYSINIT)
	/* Perform processor initialization */
	sysinit();
	
   // outSRS();
      
    cpu_identify();
#endif
    
	/* Jump to main process */
	main();

	/* No actions to perform after this so wait forever */
	while(1);
}
/********************************************************************/
/*!
 * \brief   Kinetis Identify
 * \return  None
 *
 * This is primarly a reporting function that displays information
 * about the specific CPU to the default terminal including:
 * - Kinetis family
 * - package
 * - die revision
 * - P-flash size
 * - Ram size
 */
void cpu_identify (void)
{
  uint16 temp;
    /* Determine the Kinetis family */
    switch((SIM->SDID & SIM_SDID_FAMID(0x7))>>SIM_SDID_FAMID_SHIFT) 
    {  
    	case 0x1:
    		printf("\nMKV10Zx");
    		break;
	    default:
		    printf("\nUnrecognized Kinetis family device.\n");  
		    break;  	
    }
    
    /* Determine Sub-Family ID */
    switch((SIM->SDID & SIM_SDID_SUBFAMID(0x7))>>SIM_SDID_SUBFAMID_SHIFT) 
    {  
    	case 0x0:
    		printf(" \n");
    		break;

	    default:
		    printf("\nUnrecognized Kinetis V sub-family device.\n");  
		    break;  	
    }
    
    /* Determine the package size */
    switch((SIM->SDID & SIM_SDID_PINID(0xF))>>SIM_SDID_PINID_SHIFT) 
    {  
			case 0x1:
    		break;
    	case 0x2:
    		printf("32pin       ");
    		break;
    	case 0x3:
    		printf("44pin       ");
    		break;
    	case 0x4:
    		printf("48pin      ");
    		break;

	    default:
		    printf("\nUnrecognized Kinetis V package code = 0x%x\n", (SIM->SDID & SIM_SDID_PINID(0xF))>>SIM_SDID_PINID_SHIFT);  
		   break;  	
    }
    
    /* Determine Attribute ID */
    switch((SIM->SDID & SIM_SDID_SERIERID(0x7))>>SIM_SDID_SERIERID_SHIFT) 
    {  
    	case 0x6:
    		printf("V Family - Motor control with Cortex M0+\n\n");
    		break;
	default:
		printf("\nUnrecognized Kinetis V family attribute.\n");  
		break;  	
    }
    
    /* Determine the System SRAM Size */
    switch((SIM->SDID & SIM_SDID_SRAMSIZE(0xF))>>SIM_SDID_SRAMSIZE_SHIFT) 
    {  

        case 0x3:
          printf("SRAM Size:  4 KB\n");
          break;
        case 0x4:
          printf("SRAM Size:  8 KB\n");
          break;
	   default:
		printf("\nUnrecognized SRAM Size code = 0x%x\n", (SIM->SDID & SIM_SDID_SRAMSIZE(0xF))>>SIM_SDID_SRAMSIZE_SHIFT);  
		break;  	
    }                

    /* Determine the revision ID */
    temp = ((SIM->SDID&SIM_SDID_REVID(0xF))>>SIM_SDID_REVID_SHIFT);
    printf("Silicon rev %d\n ", temp);
    
    /* Determine the flash revision */
    flash_identify();    
    
    /* Determine the P-flash size */
  switch((SIM->FCFG1 & SIM_FCFG1_PFSIZE(0xF))>>SIM_FCFG1_PFSIZE_SHIFT)
    {
  	case 0x0:
    		printf("Flash size:  8 KB program flash, 0.25 KB protection region\n");
    		break;
    	case 0x1:
    		printf("Flash size:  16 KB program flash, 0.5 KB protection region\n");
    		break;
        case 0x3:
    		printf("Flash size:  32 KB program flash, 1 KB protection region\n");
    		break;
    	case 0x5:
    		printf("Flash size:  64 KB program flash, 2 KB protection region\n");
    		break;
        case 0x7:
    		printf("Flash size:  128 KB program flash, 4 KB protection region\n");
    		break;
        case 0x9:
    		printf("Flash size:  256 KB program flash, 4 KB protection region\n");
    		break;
        case 0xF:
    		printf("Flash size:  32 KB program flash, 1 KB protection region\n");
    		break;
	default:
		printf("ERR!! Undefined flash size\n");  
		break;  	  		
    }
  
}
/********************************************************************/
/*!
 * \brief   flash Identify
 * \return  None
 *
 * This is primarly a reporting function that displays information
 * about the specific flash parameters and flash version ID for 
 * the current device. These parameters are obtained using a special
 * flash command call "read resource." The first four bytes returned
 * are the flash parameter revision, and the second four bytes are
 * the flash version ID.
 */
void flash_identify (void)
{
    /* Get the flash parameter version */

    /* Write the flash FCCOB registers with the values for a read resource command */
    FTFA->FCCOB0 = 0x03;
    FTFA->FCCOB1 = 0x00;
    FTFA->FCCOB2 = 0x00;
    FTFA->FCCOB3 = 0x00;
    FTFA->FCCOB8 = 0x01;

    /* All required FCCOBx registers are written, so launch the command */
    FTFA->FSTAT = FTFA_FSTAT_CCIF_MASK;

    /* Wait for the command to complete */
    while(!(FTFA->FSTAT & FTFA_FSTAT_CCIF_MASK));
    
    printf("Flash parameter version %d.%d.%d.%d\n",FTFA->FCCOB4,FTFA->FCCOB5,FTFA->FCCOB6,FTFA->FCCOB7);
 
    /* Get the flash version ID */   

    /* Write the flash FCCOB registers with the values for a read resource command */
    FTFA->FCCOB0 = 0x03;
    FTFA->FCCOB1 = 0x00;
    FTFA->FCCOB2 = 0x00;
    FTFA->FCCOB3 = 0x04;
    FTFA->FCCOB8 = 0x01;

    /* All required FCCOBx registers are written, so launch the command */
    FTFA->FSTAT = FTFA_FSTAT_CCIF_MASK;

    /* Wait for the command to complete */
    while(!(FTFA->FSTAT & FTFA_FSTAT_CCIF_MASK));

    printf("Flash version ID %d.%d.%d.%d\n",FTFA->FCCOB4,FTFA->FCCOB5,FTFA->FCCOB6,FTFA->FCCOB7);  
}

/********************************************************************/
/*!
 * \brief   flash SystemInit
 * \return  None
 *
 * this is a system initialization function which dediu16Cated in Keil 
 * others complier don't use it.
 * it is similar to start function
 */
void SystemInit( void )
{
	/* Disable the watchdog timer */
  WDOG_Disable();
	/* Perform processor initialization */
	sysinit();
	
	cpu_identify();
	
}


