/*
 * File:        sysinit.c
 * Purpose:     Kinetis Configuration
 *              Initializes processor to a default state
 *
 * Notes:
 *
 */

#include "common.h"
#include "sysinit.h"
#include "uart.h"
#include "mcg.h"
#include "kv10_config.h"

/********************************************************************/
/* Actual system clock frequency */
int mcg_clk_hz;
int mcg_clk_khz;
int core_clk_khz;
int periph_clk_khz;
//int pll_clk_khz;

int core_clk_mhz;

/********************************************************************/
void sysinit (void)
{
        /*
         * Enable all of the port clocks. These have to be enabled to configure
         * pin muxing options, so most code will need all of these on anyway.
         */
        SIM->SCGC5 |= (SIM_SCGC5_PORTA_MASK
                      | SIM_SCGC5_PORTB_MASK
                      | SIM_SCGC5_PORTC_MASK
                      | SIM_SCGC5_PORTD_MASK
                      | SIM_SCGC5_PORTE_MASK );
        
    // releases hold with ACKISO:  Only has an effect if recovering from VLLS1, VLLS2, or VLLS3
    // if ACKISO is set you must clear ackiso before calling pll_init 
    //    or pll init hangs waiting for OSC to initialize
    // if osc enabled in low power modes - enable it first before ack
    // if I/O needs to be maintained without glitches enable outputs and modules first before ack.

    if (PMC->REGSC &  PMC_REGSC_ACKISO_MASK)
        PMC->REGSC |= PMC_REGSC_ACKISO_MASK;

       /* Ramp up the system clock */
       /* Set the system dividers */
       /* NOTE: The PLL init will not configure the system clock dividers,
        * so they must be configured appropriately before calling the PLL
        * init function to ensure that clocks remain in valid ranges.
        */  
        SIM->CLKDIV1 = ( 0
                        | SIM_CLKDIV1_OUTDIV1(0)
                        | SIM_CLKDIV1_OUTDIV4(2) );


#if defined(FEI_72M)        
    mcg_clk_hz = 72000000L;
    MCG->C4 |= MCG_C4_DRST_DRS(2) | MCG_C4_DMX32_MASK;    /* Mid-high Frequency Range - 60C75 MHz, reference 32768 Hz,
                                                           * PLL factor 2197, 72 MHz
                                                           */
#else
    mcg_clk_hz =  fei_fee(CLK0_FREQ_HZ, LOW_POWER, CLK0_TYPE);      
#endif
    if (mcg_clk_hz < 0x100)
    while(1);
    mcg_clk_khz = mcg_clk_hz / 1000;
	core_clk_khz = mcg_clk_khz / (((SIM->CLKDIV1 & SIM_CLKDIV1_OUTDIV1_MASK) >> 28)+ 1);  	
    periph_clk_khz = mcg_clk_khz / (((SIM->CLKDIV1 & SIM_CLKDIV1_OUTDIV4_MASK) >> 16)+ 1);


  	/* Enable the pins for the selected UART */
         if (TERM_PORT == UART0)
         {
#ifdef TWR_KV10Z32
           /* Enable the UART0_TXD function on PTB17 */
            PORTB->PCR[17] = PORT_PCR_MUX(0x3); // UART is alt3 function for this pin

            /* Enable the UART0_RXD function on PTB16 */
            PORTB->PCR[16] = PORT_PCR_MUX(0x3); // UART is alt3 function for this pin    
          
#else
            /* Enable the UART0_TXD function on PTD2 */
            PORTD->PCR[2]= PORT_PCR_MUX(0x3); // UART is alt3 function for this pin

            /* Enable the UART0_RXD function on PTD3 */
            PORTD->PCR[3] = PORT_PCR_MUX(0x3); // UART is alt3 function for this pin
#endif
            //uart_init (TERM_PORT, core_clk_khz, TERMINAL_BAUD);
         }

         if (TERM_PORT == UART1)
  	 
         {
           /* Enable the UART0_TXD function on PTD0 */
            PORTD->PCR[0] = PORT_PCR_MUX(0x5); // UART is alt3 function for this pin

           /* Enable the UART0_RXD function on PTD1 */
            PORTD->PCR[1] = PORT_PCR_MUX(0x5); // UART is alt3 function for this pin 
            
            //uart_init (TERM_PORT, periph_clk_khz, TERMINAL_BAUD);
  	 
         }
      
         UART_InitPrint();
  	
     /* UART0 and UART1 are clocked from the core clock, but all other UARTs are
         * clocked from the peripheral clock. So we have to determine which clock
         * to send to the uart_init function.
         */     
  	     	
}

/*****************************************************************************//*!
+FUNCTION----------------------------------------------------------------
* @function name: UART_InitPrint
*
* @brief initialize UART for print on port defined by TERM_PORT.
*        
* @param  none 
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/

void UART_InitPrint(void)
{
    UART_ConfigType sConfig = {0};

    if (TERM_PORT == UART0)
    {
        sConfig.u32SysClkHz = core_clk_khz; 
    }
    else
    {
    
        sConfig.u32SysClkHz = periph_clk_khz;
    }
    sConfig.u32Baudrate  = TERMINAL_BAUD;
    
    UART_Init (TERM_PORT, &sConfig);
}

