/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file vectors.h
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun. 25, 2013
*
* @brief provide interrupt vector table for Kinetis E.
*
*******************************************************************************/


#ifndef __VECTORS_H
#define __VECTORS_H     1

// function prototype for default_isr in vectors.c
void default_isr(void);
void abort_isr(void);


void hard_fault_handler_c(unsigned int * hardfault_args);

/* Interrupt Vector Table Function Pointers */
typedef void pointer(void);

extern void __startup(void);
extern unsigned long __BOOT_STACK_ADDRESS[];
extern void __iar_program_start(void);
extern void SRTC_ISR(void);
extern unsigned long __initial_sp[];
extern void Reset_Handler( void );
#if (defined(__GNUC__))
extern unsigned long _estack;
extern void __thumb_startup(void);
#define VECTOR_000      (pointer*)&_estack	//          ARM core        Initial Supervisor SP
#define VECTOR_001      __thumb_startup	// 0x0000_0004 1 -          ARM core        Initial Program Counter
//#define VECTOR_001      __startup //__thumb_startup	// 0x0000_0004 1 -          ARM core        Initial Program Counter
#elif (defined(KEIL))
#define VECTOR_000      (pointer*)__initial_sp	//          ARM core        Initial Supervisor SP
#define VECTOR_001      Reset_Handler						// 0x0000_0004 1 -          ARM core        Initial Program Counter
#else
    																						// Address     Vector IRQ   Source module   Source description
#define VECTOR_000      (pointer*)__BOOT_STACK_ADDRESS	//          ARM core        Initial Supervisor SP
#define VECTOR_001      __startup	// 0x0000_0004 1 -          ARM core        Initial Program Counter
#endif
#define VECTOR_002      default_isr     // 0x0000_0008 2 -          ARM core        Non-maskable Interrupt (NMI)
#define VECTOR_003      default_isr     // 0x0000_000C 3 -          ARM core        Hard Fault
#define VECTOR_004      default_isr     // 0x0000_0010 4 -
#define VECTOR_005      default_isr     // 0x0000_0014 5 -          ARM core         Bus Fault
#define VECTOR_006      default_isr     // 0x0000_0018 6 -          ARM core         Usage Fault
#define VECTOR_007      default_isr     // 0x0000_001C 7 -                           
#define VECTOR_008      default_isr     // 0x0000_0020 8 -                           
#define VECTOR_009      default_isr     // 0x0000_0024 9 -
#define VECTOR_010      default_isr     // 0x0000_0028 10 -
#define VECTOR_011      default_isr     // 0x0000_002C 11 -         ARM core         Supervisor call (SVCall)
#define VECTOR_012      default_isr     // 0x0000_0030 12 -         ARM core         Debug Monitor
#define VECTOR_013      default_isr     // 0x0000_0034 13 -                          
#define VECTOR_014      default_isr     // 0x0000_0038 14 -         ARM core         Pendable request for system service (PendableSrvReq)
#define VECTOR_015      default_isr     // 0x0000_003C 15 -         ARM core         System tick timer (SysTick)
#define VECTOR_016      default_isr     // 0x0000_0040 16     0     DMA              DMA Channel 0 transfer complete
#define VECTOR_017      default_isr     // 0x0000_0044 17     1     DMA              DMA Channel 1 transfer complete
#define VECTOR_018      default_isr     // 0x0000_0048 18     2     DMA              DMA Channel 2 transfer complete
#define VECTOR_019      default_isr     // 0x0000_004C 19     3     DMA              DMA Channel 3 transfer complete
#define VECTOR_020      default_isr     // 0x0000_0050 20     4     DMA              DMA channel 0,1,2,3 error
#define VECTOR_021      default_isr     // 0x0000_0054 21     5     NVM              FTFA flash memory command complete,ECC fault
#define VECTOR_022      default_isr     // 0x0000_0058 22     6     PMC              LVD,LVW interrupt
#define VECTOR_023      default_isr     // 0x0000_005C 23     7     LLWU             LLWU/IRQ
#define VECTOR_024      default_isr     // 0x0000_0060 24     8     I2C0             I2C
#define VECTOR_025      default_isr     // 0x0000_0064 25     9     -                --
#define VECTOR_026      default_isr     // 0x0000_0068 26    10     SPI0             SPI0
#define VECTOR_027      default_isr     // 0x0000_006C 27    11     -                --
#define VECTOR_028      default_isr     // 0x0000_0070 28    12     UART0            UART0
#define VECTOR_029      default_isr     // 0x0000_0074 29    13     UART1            UART1
#define VECTOR_030      default_isr     // 0x0000_0078 30    14     -                --
#define VECTOR_031      default_isr     // 0x0000_007C 31    15     ADC0             --
#define VECTOR_032      default_isr     // 0x0000_0080 32    16     ADC1             -- 
#define VECTOR_033      default_isr     // 0x0000_0084 33    17     FTM0             FlexTimer0
#define VECTOR_034      default_isr     // 0x0000_0088 34    18     FTM1             FlexTimer1
#define VECTOR_035      default_isr     // 0x0000_008C 35    19     FTM2             FlexTimer2
#define VECTOR_036      default_isr     // 0x0000_0090 36    20     CMP0             ACMP0
#define VECTOR_037      default_isr     // 0x0000_0094 37    21     CMP1             ACMP1
#define VECTOR_038      default_isr     // 0x0000_0098 38    22     -                --
#define VECTOR_039      default_isr     // 0x0000_009C 39    23	    WDOG/EWM         --
#define VECTOR_040      default_isr     // 0x0000_00A0 40    24     -                --
#define VECTOR_041      default_isr     // 0x0000_00A4 41    25     DAC0             --
#define VECTOR_042      default_isr     // 0x0000_00A8 42    26     -                --
#define VECTOR_043      default_isr     // 0x0000_00AC 43    27     MCG              MCG loss of lock
#define VECTOR_044      default_isr     // 0x0000_00B0 44    28     LPTMR0           --
#define VECTOR_045      default_isr     // 0x0000_00B4 45    29     PDB              --
#define VECTOR_046      default_isr     // 0x0000_00B8 46    30     Port A         
#define VECTOR_047      default_isr     // 0x0000_00BC 47    31     PORT B,C,D,E     // END of real vector table

#ifndef USE_BOOTLOADER
#define CONFIG_1		0xffffffff 
#define CONFIG_2		0xffffffff 
#define CONFIG_3		0xffffffff
#define CONFIG_4		0xfffffffe
#endif
#endif /*__VECTORS_H*/

/* End of "vectors.h" */
