/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file adc.c
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun. 25, 2013
*
* @brief providing APIs for configuring ADC module (ADC). 
*
*******************************************************************************
*
* provide APIs for configuring ADC module (ADC)
******************************************************************************/
#include "common.h"
#include "adc.h"
/******************************************************************************
* Local function
******************************************************************************/
ADC_CallbackType ADC_Callback[2] = {NULL};
/******************************************************************************
* Local variables
******************************************************************************/

/******************************************************************************
* Local function prototypes
******************************************************************************/

/******************************************************************************
* define ADC APIs
*
*//*! @addtogroup adc_api_list
* @{
*******************************************************************************/

/*****************************************************************************//**
   *
   * @brief initialize ADC module.
   *
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
uint8_t ADC_Cal(ADC_Type *pADC)
{

  unsigned short cal_var;
    
  /* Sanity check */
  ASSERT((pADC == ADC0) || (pADC == ADC1));
  ASSERT((pADC == ADC0) || (pADC == ADC1));
  if( pADC == ADC0)
    
  {    
      SIM->SCGC6 |= SIM_SCGC6_ADC0_MASK; 
  }    
  else
    
  {  
      SIM->SCGC6 |= SIM_SCGC6_ADC1_MASK;
  }    
  
  pADC->SC2 &=  ~ADC_SC2_ADTRG_MASK ; // Enable Software Conversion Trigger for Calibration Process    - ADC0_SC2 = ADC0_SC2 | ADC_SC2_ADTRGW(0);   
  pADC->SC3 &= ( ~ADC_SC3_ADCO_MASK & ~ADC_SC3_AVGS_MASK ); // set single conversion, clear avgs bitfield for next writing
  pADC->SC3 |= ( ADC_SC3_AVGE_MASK | ADC_SC3_AVGS(AVGS_32) );  // Turn averaging ON and set at max value ( 32 )
  
  
  pADC->SC3 |= ADC_SC3_CAL_MASK ;      // Start CAL
  while (( pADC->SC1[0] & ADC_SC1_COCO_MASK ) == COCO_NOT ); // Wait calibration end
  	
  if (( pADC->SC3 & ADC_SC3_CALF_MASK) == CALF_FAIL )
  {    
      return(1);    // Check for Calibration fail error and return   
       
  }
  // Calculate plus-side calibration
  cal_var = 0x00;
  
  cal_var =   pADC->CLP0; 
  cal_var +=  pADC->CLP1;
  cal_var +=  pADC->CLP2;
  cal_var +=  pADC->CLP3;
  cal_var +=  pADC->CLP4;
  cal_var +=  pADC->CLPS;

  cal_var = cal_var/2;
  cal_var |= 0x8000; // Set MSB

  pADC->PG = ADC_PG_PG(cal_var);
 

  // Calculate minus-side calibration
  cal_var = 0x00;

  cal_var =  pADC->CLM0; 
  cal_var += pADC->CLM1;
  cal_var += pADC->CLM2;
  cal_var += pADC->CLM3;
  cal_var += pADC->CLM4;
  cal_var += pADC->CLMS;

  cal_var = cal_var/2;

  cal_var |= 0x8000; // Set MSB

  pADC->MG = ADC_MG_MG(cal_var); 
  
  pADC->SC3 &= ~ADC_SC3_CAL_MASK ; /* Clear CAL bit */

  return(0);
}
/*****************************************************************************//**
   *
   * @brief initialize ADC module.
   *
   * @param[in]  pADC point to ADC module type. 
   * @param[in]  pADC_Config point to ADC configuration structure. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
void ADC_Init(ADC_Type *pADC, ADC_ConfigTypePtr pADC_Config)
{
   ASSERT((pADC == ADC0) || (pADC == ADC1));
    if( pADC == ADC0)
    {
        SIM->SCGC6 |= SIM_SCGC6_ADC0_MASK;
    }
    else
    {
        SIM->SCGC6 |= SIM_SCGC6_ADC1_MASK;
    }    

    /* set clock cource for ADC */
    ADC_SelectClock(pADC,pADC_Config->bClocklSelect);

    /* set clock divide */
    ADC_SelectClockDivide(pADC,pADC_Config->bClockDivider);

    /* set ADC mode */
    ADC_SetMode(pADC,pADC_Config->bConversionMode);

    if( pADC_Config->bCompareEn )
    {
        ADC_CompareEnable(pADC);
    }
    
    if( pADC_Config->bCompareGreaterEn )
    {
        ADC_CompareGreaterFunction(pADC);
    }
    
    if( pADC_Config->bAverageEn )
    {
        pADC->SC3 &= ~(ADC_SC3_AVGE_MASK | ADC_SC3_AVGS_MASK);
        pADC->SC3 |= ADC_SC3_AVGE_MASK | ADC_SC3_AVGS(pADC_Config->bAverageSelect);
    }
    else
    {
      
        pADC->SC3 &= ~(ADC_SC3_AVGE_MASK | ADC_SC3_AVGS_MASK);
    }
    if(pADC_Config->bDMAEn)
    {
       pADC->SC2 |= ADC_SC2_DMAEN_MASK; 
    }
    if( pADC_Config->bContinuousEn )
    {
        ADC_ContinuousConversion(pADC);
    }

    if( pADC_Config->bHardwareTriggerEn )
    {
        ADC_SetHardwareTrigger(pADC);
    }

    if( pADC_Config->bIntEn )
    {
        ADC_IntEnable(pADC, 0);
        ADC_IntEnable(pADC, 1);
        if (pADC ==ADC0)
        {
            NVIC_EnableIRQ( ADC0_IRQn );
        }
        else
        {
            NVIC_EnableIRQ( ADC1_IRQn );
        }
    } 
 
    if( pADC_Config->bDiffEn )
    {
        ADC_DiffModeEnable(pADC, 0 );  
    }
    if( pADC_Config->bLongSampleEn )
    {
        ADC_SetLongSample(pADC);
    } 

    if( pADC_Config->bLowPowerEn )
    {
        ADC_SetLowPower (pADC);
    }
}

/*****************************************************************************//*!
   *
   * @brief disable ADC module.
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none.
   *
   * @ Pass/ Fail criteria: none.
   *****************************************************************************/
void ADC_DeInit( ADC_Type *pADC, uint8_t u8RegIndex)
{
    ADC_SetChannel(pADC, u8RegIndex, ADC_CHANNEL_DISABLE, FALSE);

    SIM->SCGC6 &= ~SIM_SCGC6_ADC0_MASK;
    SIM->SCGC6 &= ~SIM_SCGC6_ADC1_MASK;    
}

/*****************************************************************************//*!
   *
   * @brief start a conversion and get conversion result
   *        
   * @param[in]  pADC point to ADC module type. 
   * @param[in]  u8Channel adc channel to conversion. 
   *
   * @return ADC conversion result.
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
unsigned int ADC_PollRead( ADC_Type *pADC, uint8_t u8RegIndex, uint8_t u8Channel )
{
		ADC_SetChannel(pADC, u8RegIndex, u8Channel, FALSE);
		while( !ADC_IsCOCOFlag(pADC, u8RegIndex ) );
		return ADC_ReadResultReg(pADC, u8RegIndex);
}


/*****************************************************************************//*!
   *
   * @brief install ADC call back function.
   *        
   * @param[in]	 pADC_CallBack point to address of  adc call back function.
   *
   * @return none.
   *
   * @ Pass/ Fail criteria: none.
   *****************************************************************************/
void ADC_SetCallBack(uint8_t ADCIndex, ADC_CallbackType pADC_CallBack)
{
    ADC_Callback[ADCIndex] = pADC_CallBack;
}

/*****************************************************************************//*!
   *
   * @brief set ADC channel.
   *        
   * @param[in]  pADC point to ADC module type. 
   * @param[in]  u8RegIndex adc channel ping-poing index. 
   * @param[in]  u8Channel adc channel to conversion. 
   * @param[in]  u8IsDiff 1 if in differential input, 0 otherwise. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
void ADC_SetChannel( ADC_Type *pADC, uint8_t u8RegIndex, uint8_t u8Channel, uint8_t u8IsDiff )
{
    uint32_t u32temp;    
    u32temp = pADC->SC1[u8RegIndex]; 
    u32temp &= ~ADC_SC1_ADCH_MASK;
    if(u8IsDiff)
    {
        u32temp |= ADC_SC1_DIFF_MASK;       /* differential input */
    }
    else
    {
        u32temp &= ~ADC_SC1_DIFF_MASK;      /* single endede input */     
    }
    pADC->SC1[u8RegIndex] = u32temp|ADC_SC1_ADCH(u8Channel);   
}
/*****************************************************************************//*!
   *
   * @brief Voltage Reference Selection.
   *        
   * @param[in]  pADC point to ADC module type. 
   * @param[in]  u8Vref adc reference voltage selection. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
void ADC_VrefSelect( ADC_Type *pADC, uint8_t u8Vref )
{
    uint32_t u32Temp;
    u32Temp = pADC->SC2;
    u32Temp &= ~ADC_SC2_REFSEL_MASK;
    pADC->SC2 = u32Temp|ADC_SC2_REFSEL(u8Vref);
}

/*****************************************************************************//*!
   *
   * @brief select clock divide
   *        
   * @param[in]  pADC point to ADC module type. 
   * @param[in]  u8Div Clock Divide Select. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
void ADC_SelectClockDivide( ADC_Type *pADC, uint8_t u8Div )
{
    uint32_t u32Temp;
    u32Temp = pADC->CFG1;
    u32Temp &= ~ADC_CFG1_ADIV_MASK;
    pADC->CFG1 = u32Temp|ADC_CFG1_ADIV(u8Div);
}

/*****************************************************************************//*!
   *
   * @brief set ADC mode.
   *        
   * @param[in]  pADC point to ADC module type. 
   * @param[in]  u8Mode Conversion Mode Selection. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
void ADC_SetMode( ADC_Type *pADC, uint8_t u8Mode )
{
    uint32_t u32Temp;
    u32Temp = pADC->CFG1;
    u32Temp &= ~ADC_CFG1_MODE_MASK;
    pADC->CFG1= u32Temp|ADC_CFG1_MODE(u8Mode);
}
/*****************************************************************************//*!
   *
   * @brief Input Clock Select.
   *        
   * @param[in]  pADC point to ADC module type. 
   * @param[in]  u8Clock Input Clock Select. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
void ADC_SelectClock( ADC_Type *pADC, uint8_t u8Clock )
{
    uint32_t u32Temp;
    u32Temp = pADC->CFG1;
    u32Temp &= ~ADC_CFG1_ADICLK_MASK;
    pADC->CFG1 = u32Temp|ADC_CFG1_ADICLK(u8Clock);
}


/*! @} End of adc_api_list                                               						*/


/*****************************************************************************//*!
   *
   * @brief ADC interrupt service routine.
   *        
   * @param  none. 
   *
   * @return none.
   *
   * @ Pass/ Fail criteria: none.
   *****************************************************************************/
void ADC0_Isr(void)
{
    if( ADC_Callback[0] )
    {
        ADC_Callback[0]();
    }

}

void ADC1_Isr(void)
{
 
    if( ADC_Callback[1] )
    {
        ADC_Callback[1]();
    }
   
}
