/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file adc.h
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun. 25, 2013
*
* @brief header file for Analog-to-Digital Converter (ADC). 
*
*******************************************************************************
*
* provide APIs for accessing ADC module (ADC)
******************************************************************************/

#ifndef _ADC_H_
#define _ADC_H_
#ifdef __cplusplus
extern "C" {
#endif
/******************************************************************************
* Macros
******************************************************************************/
/******************************************************************************
*define ADC refernce voltage 
*
*//*! @addtogroup adc_ref_list
* @{
*******************************************************************************/

#define ADC_VREF_VREFH                 0x00			/*!< ADC reference voltage is VREFH*/
#define ADC_VREF_VDDA                  0x01			/*!< ADC reference voltage is VDDA*/

/*! @} End of adc_ref_list                                                    						*/

/******************************************************************************
* define ADC clock source 
*
*//*! @addtogroup adc_clock_source_list
* @{
*******************************************************************************/

#define CLOCK_SOURCE_BUS_CLOCK							0x00	/*!< ADC clock source is bus clock*/
#define CLOCK_SOURCE_BUS_CLOCK_DIVIDE_2			        0x01	/*!< ADC clock source is bus clock devided by 2*/
#define CLOCK_SOURCE_ALTCLK								0x02	/*!< ADC clock source is alternative clcok*/
#define CLOCK_SOURCE_ADACK								0x03	/*!< ADC clock source is asynchronous clock*/
/*! @} End of adc_clock_source_list                                          						*/


/******************************************************************************
* define ADC divider 
*
*//*! @addtogroup adc_clock_divider_list
* @{
*******************************************************************************/

#define ADC_ADIV_DIVIDE_1								0x00		/*!< ADC clock divide by 1*/
#define ADC_ADIV_DIVIDE_2								0x01		/*!< ADC clock divide by 2*/
#define ADC_ADIV_DIVIDE_4								0x02		/*!< ADC clock divide by 4*/
#define ADC_ADIV_DIVIDE_8								0x03		/*!< ADC clock divide by 8*/
/*! @} End of adc_clock_divider_list                                          						*/

/******************************************************************************
* define ADC mode 
*
*//*! @addtogroup adc_mode_list
* @{
*******************************************************************************/

#define ADC_MODE_8BIT									0x00		/*!< ADC 8bit mode*/
#define ADC_MODE_10BIT									0x01		/*!< ADC 10bit mode*/
#define ADC_MODE_12BIT									0x02		/*!< ADC 12bit mode */
/*! @} End of adc_mode_list                                               						*/

/******************************************************************************
* define ADC channel
*
*//*! @addtogroup adc_channel_list
* @{
*******************************************************************************/

#define ADC_CHANNEL_AD0                                 0x0 /*!< ADC input channel 0 */
#define ADC_CHANNEL_AD1                                 0x1 /*!< ADC input channel 1 */
#define ADC_CHANNEL_AD2                                 0x2 /*!< ADC input channel 2 */
#define ADC_CHANNEL_AD3                                 0x3 /*!< ADC input channel 3 */
#define ADC_CHANNEL_AD4                                 0x4 /*!< ADC input channel 4 */
#define ADC_CHANNEL_AD5                                 0x5 /*!< ADC input channel 5 */
#define ADC_CHANNEL_AD6                                 0x6 /*!< ADC input channel 6 */
#define ADC_CHANNEL_AD7                                 0x7 /*!< ADC input channel 7 */
#define ADC_CHANNEL_AD8                                 0x8 /*!< ADC input channel 8 */
#define ADC_CHANNEL_AD9                                 0x9 /*!< ADC input channel 9 */
#define ADC_CHANNEL_AD10                                 0xa /*!< ADC input channel 10 */
#define ADC_CHANNEL_AD11                                 0xb /*!< ADC input channel 11 */
#define ADC_CHANNEL_AD12                                 0xc /*!< ADC input channel 12 */
#define ADC_CHANNEL_AD13                                 0xd /*!< ADC input channel 13 */
#define ADC_CHANNEL_AD14                                 0xe /*!< ADC input channel 14 */
#define ADC_CHANNEL_AD15                                 0xf /*!< ADC input channel 15 */
#define ADC_CHANNEL_AD18_VSS                            0x12 /*!< ADC input channel VSS */ 
#define ADC_CHANNEL_AD22_TEMPSENSOR                     0x16 /*!< ADC input channel internal temperature sensor */
#define ADC_CHANNEL_AD23_BANDGAP                        0x17 /*!< ADC input channel bandgap */ 
#define ADC_CHANNEL_AD29_VREFH                          0x1D /*!< ADC input channel Vrefh */ 
#define ADC_CHANNEL_AD30_VREFL                          0x1E /*!< ADC input channel Vrefl */ 
#define ADC_CHANNEL_DISABLE                             0x1F /*!< ADC disable */
/*! @} End of adc_channel_list                                               						*/

#define ADC_COMPARE_LESS                                0x00
#define ADC_COMPARE_GREATER                             0x01

#define A                 0x0
#define B                 0x1


/////// NOTE: the following defines relate to the ADC register definitions
/////// and the content follows the reference manual, using the same symbols.


//// ADCSC1 (register)

// Conversion Complete (COCO) mask
#define COCO_COMPLETE     ADC_SC1_COCO_MASK
#define COCO_NOT          0x00

// ADC interrupts: enabled, or disabled.
#define AIEN_ON           ADC_SC1_AIEN_MASK
#define AIEN_OFF          0x00

// Differential or Single ended ADC input
#define DIFF_SINGLE       0x00
#define DIFF_DIFFERENTIAL ADC_SC1_DIFF_MASK

//// ADCCFG1

// Power setting of ADC
#define ADLPC_LOW         ADC_CFG1_ADLPC_MASK
#define ADLPC_NORMAL      0x00

// Clock divisor
#define ADIV_1            0x00
#define ADIV_2            0x01
#define ADIV_4            0x02
#define ADIV_8            0x03

// Long samle time, or Short sample time
#define ADLSMP_LONG       ADC_CFG1_ADLSMP_MASK
#define ADLSMP_SHORT      0x00

// How many bits for the conversion?  8, 12, 10, or 16 (single ended).
#define MODE_8            0x00
#define MODE_12           0x01
#define MODE_10           0x02
#define MODE_16           0x03

// ADC Input Clock Source choice? Bus clock, Bus clock/2, "altclk", or the 
//                                ADC's own asynchronous clock for less noise
#define ADICLK_BUS        0x00
#define ADICLK_BUS_2      0x01
#define ADICLK_ALTCLK     0x02
#define ADICLK_ADACK      0x03

//// ADCCFG2

// Select between B or A channels
#define MUXSEL_ADCB       ADC_CFG2_MUXSEL_MASK
#define MUXSEL_ADCA       0x00

// Ansync clock output enable: enable, or disable the output of it
#define ADACKEN_ENABLED   ADC_CFG2_ADACKEN_MASK
#define ADACKEN_DISABLED  0x00

// High speed or low speed conversion mode
#define ADHSC_HISPEED     ADC_CFG2_ADHSC_MASK
#define ADHSC_NORMAL      0x00

// Long Sample Time selector: 20, 12, 6, or 2 extra clocks for a longer sample time
#define ADLSTS_20          0x00
#define ADLSTS_12          0x01
#define ADLSTS_6           0x02
#define ADLSTS_2           0x03

////ADCSC2

// Read-only status bit indicating conversion status
#define ADACT_ACTIVE       ADC_SC2_ADACT_MASK
#define ADACT_INACTIVE     0x00

// Trigger for starting conversion: Hardware trigger, or software trigger.
// For using PDB, the Hardware trigger option is selected.
#define ADTRG_HW           ADC_SC2_ADTRG_MASK
#define ADTRG_SW           0x00

// ADC Compare Function Enable: Disabled, or Enabled.
#define ACFE_DISABLED      0x00
#define ACFE_ENABLED       ADC_SC2_ACFE_MASK

// Compare Function Greater Than Enable: Greater, or Less.
#define ACFGT_GREATER      ADC_SC2_ACFGT_MASK
#define ACFGT_LESS         0x00

// Compare Function Range Enable: Enabled or Disabled.
#define ACREN_ENABLED      ADC_SC2_ACREN_MASK
#define ACREN_DISABLED     0x00

// DMA enable: enabled or disabled.
#define DMAEN_ENABLED      ADC_SC2_DMAEN_MASK
#define DMAEN_DISABLED     0x00

// Voltage Reference selection for the ADC conversions
// (***not*** the PGA which uses VREFO only).
// VREFH and VREFL (0) , or VREFO (1).

#define REFSEL_EXT         0x00
#define REFSEL_ALT         0x01
#define REFSEL_RES         0x02     /* reserved */
#define REFSEL_RES_EXT     0x03     /* reserved but defaults to Vref */

////ADCSC3

// Calibration begin or off
#define CAL_BEGIN          ADC_SC3_CAL_MASK
#define CAL_OFF            0x00

// Status indicating Calibration failed, or normal success
#define CALF_FAIL          ADC_SC3_CALF_MASK
#define CALF_NORMAL        0x00

// ADC to continously convert, or do a sinle conversion
#define ADCO_CONTINUOUS    ADC_SC3_ADCO_MASK
#define ADCO_SINGLE        0x00

// Averaging enabled in the ADC, or not.
#define AVGE_ENABLED       ADC_SC3_AVGE_MASK
#define AVGE_DISABLED      0x00

// How many to average prior to "interrupting" the MCU?  4, 8, 16, or 32
#define AVGS_4             0x00
#define AVGS_8             0x01
#define AVGS_16            0x02
#define AVGS_32            0x03

////PGA

// PGA enabled or not?
#define PGAEN_ENABLED      ADC_PGA_PGAEN_MASK
#define PGAEN_DISABLED     0x00 

// Chopper stabilization of the amplifier, or not.
#define PGACHP_CHOP        ADC_PGA_PGACHP_MASK 
#define PGACHP_NOCHOP      0x00

// PGA in low power mode, or normal mode.
#define PGALP_LOW          ADC_PGA_PGALP_MASK
#define PGALP_NORMAL       0x00

// Gain of PGA.  Selectable from 1 to 64.
#define PGAG_1             0x00
#define PGAG_2             0x01
#define PGAG_4             0x02
#define PGAG_8             0x03
#define PGAG_16            0x04
#define PGAG_32            0x05
#define PGAG_64            0x06

/******************************************************************************
* define ADC call back
*
*//*! @addtogroup adc_callback
* @{
*******************************************************************************/
typedef void (*ADC_CallbackType)(void);							/*!< ADC call back function */
/*! @} End of adc_callback                                               						*/

/******************************************************************************
* 
*
*//*! @addtogroup adc_config_type  
* @{
*******************************************************************************/
/*!
 * @brief ADC configuration type.
 *
 */

typedef struct
{
    /* ADCx_SC1n register setting bit */
    uint32_t bChannelSelect         :5;     /*!< 1: input channel select */
    uint32_t bDiffEn                :1 ;     /*!< 1: differential mode Enable, 0: differential mode disable */
    uint32_t bIntEn                 :1;     /*!< 1: Interrupt Enable, 0: Interrupt disable */    
    
    /*ADCx_CFG1 register setting bit */ 
    uint32_t bClocklSelect          :2;     /*!< 1: input clock select */ 
    uint32_t bConversionMode        :2;     /*!< 1: conversion mode(8 bit,12 bit,10 bit,16 bit)select */    
    uint32_t bLongSampleEn          :1;     /*!< 1: long sample mode, 0: short sample mode */
    uint32_t bClockDivider          :2;     /*!< 1: long sample mode, 0: short sample mode */    
    uint32_t bLowPowerEn            :1;     /*!< 1: Low power mode, 0: high speed mode */
    
    /*ADCx_CFG2 register setting bit */ 
    uint32_t bSampleTime            :2;      /*!< long sample time select */
    uint32_t bHighSpeedEn           :1;      /*!< 1: High-speed conversion sequence selected, 0: Normal conversion sequence selected */    
    uint32_t bAsynClockEn           :1;      /*!< 1: asynchrounous clock enable, 0: asynchrounous clock disable */
    uint32_t bMuxSelect             :1;      /*!< 1: ADxxa channel select, 0: ADxxb channel select */
    
   /*ADCx_SC2 register setting bit */    
    uint32_t bRefSelect             :2;      /*!< reference voltage select */ 
    uint32_t bDMAEn                 :1;      /*!< 1: DMA request is enabled , 0:  DMA request is disabled */ 
    uint32_t bCompareRangeEn        :1;     /*!< 1: Compare function ranege is enabled, 0: Compare function ranege is disalbed */
    uint32_t bCompareGreaterEn      :1;     /*!< 1: Compare greater mode, 0: compare less than mode */
    uint32_t bCompareEn             :1;     /*!< 1: compare function Enable, 0: compare function disable */
    uint32_t bHardwareTriggerEn     :1;     /*!< 1: hardware trigger, 0: software trigger */
    
   /*ADCx_SC3 register setting bit */ 
    uint32_t bAverageSelect         :2;      /*!< average select */      
    uint32_t bAverageEn             :1;      /*!< 1: enable hardware average , 0:  disable hardware average  */      
    uint32_t bContinuousEn          :1;     /*!< 1: Continuous Conversion Enable, 0: Continuous Conversion disable */

   /*ADCx_SC3 register setting bit */
    uint32_t bPGAOffsetMeasurementEn :1;     /*!< 1:PGA runs in offset measurement mode, 0: PGA runs in normal operation. */ 
    uint32_t bPGAGain                :4;     /*!< 1:PGA runs in offset measurement mode, 0: PGA runs in normal operation. */
    uint32_t bPGALowPowerEn          :1;     /*!< 1:PGA runs in normal power mode, 0: PGA runs in low power mode */ 
    uint32_t bPGAChoppingEn          :1;     /*!< 1:disable chopping, 0: disable  */     
    uint32_t bPGAEn                  :1;     /*!< 1:enable, 0: disable PGA */    
    
}ADC_ConfigType, *ADC_ConfigTypePtr;
/*! @} End of adc_config_type                                               						*/

/******************************************************************************
* define ADC APIs
*
*//*! @addtogroup adc_api_list
* @{
*******************************************************************************/
/*****************************************************************************//*!
   *
   * @brief enable ADC interrupt.
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_IntEnable( ADC_Type *pADC, uint8_t u8RegIndex )
{
    pADC->SC1[u8RegIndex] |= ADC_SC1_AIEN_MASK; 
    

}
/*****************************************************************************//*!
   *
   * @brief disable ADC interrupt.
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_IntDisable( ADC_Type *pADC, uint8_t u8RegIndex )
{
   pADC->SC1[u8RegIndex] &= ~ADC_SC1_AIEN_MASK;   
}

/*****************************************************************************//*!
   *
   * @brief enable ADC differential mode..
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_DiffModeEnable( ADC_Type *pADC, uint8_t u8RegIndex )
{
   pADC->SC1[u8RegIndex] |= ADC_SC1_DIFF_MASK;   
}

/*****************************************************************************//*!
   *
   * @brief disable ADC differential mode.
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_DiffModeDisable( ADC_Type *pADC, uint8_t u8RegIndex )
{
    pADC->SC1[u8RegIndex] &= ~ADC_SC1_DIFF_MASK;        
}

/*****************************************************************************//*!
   *
   * @brief enable ADC continuous conversion.
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_ContinuousConversion( ADC_Type *pADC )
{
    pADC->SC3 |= ADC_SC3_ADCO_MASK;   
}
/*****************************************************************************//*!
   *
   * @brief enable ADC single conversion
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_SingleConversion( ADC_Type *pADC )
{
    pADC->SC3 &= ~ADC_SC3_ADCO_MASK;   
}

/*****************************************************************************//*!
   *
   * @brief set the ADC to hardware trigger.
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none.
   *
   * @ Pass/ Fail criteria: none.
   *****************************************************************************/
__STATIC_INLINE void ADC_SetHardwareTrigger( ADC_Type *pADC )
{
    pADC->SC2 |= ADC_SC2_ADTRG_MASK;
}

/*****************************************************************************//*!
   *
   * @brief set the ADC to software trigger.
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_SetSoftwareTrigger( ADC_Type *pADC )
{
    pADC->SC2 &= ~ADC_SC2_ADTRG_MASK;
}

/*****************************************************************************//*!
   *
   * @brief enable ADC compare function.
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_CompareEnable( ADC_Type *pADC )
{
    pADC->SC2 |= ADC_SC2_ACFE_MASK;
}

/*****************************************************************************//*!
   *
   * @brief disable ADC compare function.
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_CompareDisable( ADC_Type *pADC )
{
    pADC->SC2 &= ~ADC_SC2_ACFE_MASK;
}

/*****************************************************************************//*!
   *
   * @brief enable ADC compare greater function.
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_CompareGreaterFunction( ADC_Type *pADC )
{
    pADC->SC2 |= ADC_SC2_ACFGT_MASK;
}

/*****************************************************************************//*!
   *
   * @brief enable ADC compare less function.
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_CompareLessFunction( ADC_Type *pADC)
{
    pADC->SC2 &= ~ADC_SC2_ACFGT_MASK;
}

/*****************************************************************************//*!
   *
   * @brief set ADC to low power configuration.
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_SetLowPower( ADC_Type *pADC)
{
    pADC->CFG1 |= ADC_CFG1_ADLPC_MASK;
}

/*****************************************************************************//*!
   *
   * @brief set ADC to high speed configuration.
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_SetHighSpeed( ADC_Type *pADC )
{
    pADC->CFG2 &= ~ADC_CFG2_ADHSC_MASK;
}

/*****************************************************************************//*!
   *
   * @brief Long Sample Time Configuration.
   *        
   * @param[in]  pADC point to ADC module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_SetLongSample( ADC_Type *pADC )
{
    pADC->CFG1 |= ADC_CFG1_ADLSMP_MASK;
}

/*****************************************************************************//*!
   *
   * @brief Short Sample Time Configuration.
   *        
   * @param[in]  pADC point to ADC module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_SetShortSample( ADC_Type *pADC )
{
    pADC->CFG1 &= ~ADC_CFG1_ADLSMP_MASK;
}

/*****************************************************************************//*!
   *
   * @brief hardware average Configuration.
   *        
   * @param[in]  pADC point to ADC module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_SetHardwareAverage( ADC_Type *pADC )
{
     pADC->SC3 |= ADC_SC3_AVGE_MASK; 
}

/*****************************************************************************//*!
   *
   * @brief DMA request Configuration.
   *        
   * @param[in]  pADC point to ADC module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_SetDMARequest( ADC_Type *pADC )
{
     pADC->SC2 |= ADC_SC2_DMAEN_MASK; 
}

/*****************************************************************************//*!
   *
   * @brief compare range Configuration.
   *        
   * @param[in]  pADC point to ADC module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_SetCompareRange( ADC_Type *pADC )
{
     pADC->SC2 |= ADC_SC2_ACREN_MASK; 
}

/*****************************************************************************//*!
   *
   * @brief compare range Configuration.
   *        
   * @param[in]  pADC point to ADC module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_SetMuxSelect( ADC_Type *pADC )
{
     pADC->CFG2 |= ADC_CFG2_MUXSEL_MASK; 
}

/*****************************************************************************//*!
   *
   * @brief asynchronous clock Configuration.
   *        
   * @param[in]  pADC point to ADC module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_SetAsynClock( ADC_Type *pADC )
{
     pADC->CFG2 |= ADC_CFG2_ADACKEN_MASK; 
}

/*****************************************************************************//*!
   *
   * @brief read ADC result register.
   *        
   * @param[in]  pADC point to ADC module type.
   *
   * @return ADC result value.
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE uint16_t ADC_ReadResultReg( ADC_Type *pADC, uint8_t u8RegIndex )
{
    return (uint16_t)pADC->R[u8RegIndex];
}
/*****************************************************************************//*!
   *
   * @brief set ADC compare value.
   *        
   * @param[in]  pADC point to ADC module type. 
   * @param[in]  u16Compare compare value. 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE void ADC_SetCompareValue( ADC_Type *pADC, uint16_t u16Compare1, uint16_t u16Compare2 )
{
    pADC->CV1 = u16Compare1;
    pADC->CV1 = u16Compare2;
}

/*****************************************************************************//*!
   *
   * @brief check conversion active status
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return TRUE or FALSE
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE uint8_t ADC_IsConversionActiveFlag( ADC_Type *pADC, uint8_t u8RegIndex )
{
    return(pADC->SC2 & ADC_SC2_ADACT_MASK);
}
/*****************************************************************************//*!
   *
   * @brief check COCO flag
   *        
   * @param[in]  pADC point to ADC module type. 
   *
   * @return TRUE or FALSE
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
__STATIC_INLINE uint8_t ADC_IsCOCOFlag( ADC_Type *pADC, uint8_t u8RegIndex )
{
    return(pADC->SC1[u8RegIndex] & ADC_SC1_COCO_MASK);
}

/******************************************************************************
* Global function
******************************************************************************/
uint8_t ADC_Cal(ADC_Type *pADC);
void ADC_SetChannel( ADC_Type *pADC, uint8_t u8RegIndex, uint8_t u8Channel, uint8_t u8IsDiff );
void ADC_IntEnable( ADC_Type *pADC, uint8_t u8RegIndex );
void ADC_IntDisable( ADC_Type *pADC, uint8_t u8RegIndex );
void ADC_ContinuousConversion( ADC_Type *pADC);
void ADC_SingleConversion( ADC_Type *pADC);
void ADC_DiffModeDisable( ADC_Type *pADC, uint8_t u8RegIndex );
void ADC_SetSoftwareTrigger( ADC_Type *pADC);
void ADC_SetHardwareTrigger( ADC_Type *pADC);
void ADC_VrefSelect( ADC_Type *pADC, uint8_t u8Vref );
void ADC_CompareEnable( ADC_Type *pADC );
void ADC_CompareDisable( ADC_Type *pADC );
void ADC_CompareGreaterFunction( ADC_Type *pADC );
void ADC_CompareLessFunction( ADC_Type *pADC );
void ADC_SetLowPower( ADC_Type *pADC );
void ADC_SetHighSpeed( ADC_Type *pADC );
void ADC_SelectClockDivide( ADC_Type *pADC, uint8_t u8Div );
void ADC_SetLongSample( ADC_Type *pADC );
void ADC_SetShortSample( ADC_Type *pADC );
void ADC_SetMode( ADC_Type *pADC, uint8_t u8Mode );
void ADC_SelectClock( ADC_Type *pADC, uint8_t u8Clock );
uint16_t ADC_ReadResultReg( ADC_Type *pADC, uint8_t u8RegIndex );
void ADC_SetCompareValue( ADC_Type *pADC, uint16_t u16Compare1, uint16_t u16Compare2 );
uint8_t ADC_IsConversionActiveFlag( ADC_Type *pADC, uint8_t u8RegIndex );
uint8_t ADC_IsCOCOFlag( ADC_Type *pADC, uint8_t u8RegIndex );
void ADC_Init(ADC_Type *pADC, ADC_ConfigTypePtr pADC_Config);
void ADC_SetCallBack(uint8_t ADCIndex, ADC_CallbackType pADC_CallBack);
void ADC_DeInit( ADC_Type *pADC, uint8_t u8RegIndex);
void ADC_SetAsynClock( ADC_Type *pADC );
void ADC_SetMuxSelect( ADC_Type *pADC );
/*! @} End of adc_api_list                                               						*/

#ifdef __cplusplus
}
#endif
#endif /* _ADC_H_ */
