
/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file CMP.c
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun. 25, 2013
*
* @brief providing APIs for configuring CMP. 
*
*******************************************************************************
*
* provide APIs for configuring CMP
******************************************************************************/
#include "common.h"
#include "cmp.h"
/******************************************************************************
* Global variables
******************************************************************************/

/******************************************************************************
* Constants and macros
******************************************************************************/

/******************************************************************************
* Local types
******************************************************************************/

/******************************************************************************
* Local function prototypes
******************************************************************************/

/******************************************************************************
* Local variables
******************************************************************************/

CMP_CallbackType CMP_Callback[2] = {NULL};

/******************************************************************************
* Local functions
******************************************************************************/
/******************************************************************************
* Global functions
******************************************************************************/
void CMP0_Isr(void);
void CMP1_Isr(void);

/******************************************************************************
* CMP api list.
*
*//*! @addtogroup CMP_api_list
* @{
*******************************************************************************/

/*****************************************************************************//*!
*
* @brief initialize CMP as per control field.
*        
* @param   pCMP         pointer to an CMP register base.
* @param   pConfig        control parameters.  
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see   CMP_DeInit.  
*
*****************************************************************************/
void CMP_Init(CMP_Type *pCMP, CMP_ConfigType *pConfig) 
{        
    /* enable CMP clock gate */
    SIM->SCGC4 |= SIM_SCGC4_CMP_MASK;
        


    if(pConfig->bFallingIntEn || pConfig->bRisingIntEn ) 
    {
        if(pCMP==CMP0)
        {
            NVIC_EnableIRQ( CMP0_IRQn );
        }
        else     
        {
            NVIC_EnableIRQ( CMP1_IRQn );
        }
    }

    if(pConfig->bFallingIntEn) 
    {
        pCMP->SCR |= CMP_SCR_IEF_MASK;
    } 
   
    if(pConfig->bRisingIntEn) 
    {
        pCMP->SCR |= CMP_SCR_IER_MASK;
    }
        
    if(pConfig->bCMPOEn)
    {
        pCMP->CR1 |=CMP_CR1_OPE_MASK; 
    }

    if(pConfig->bCMPOSelect)
    {
        pCMP->CR1 |=CMP_CR1_COS_MASK; 
    }    
    else
    {
       pCMP->CR1 &= ~CMP_CR1_COS_MASK;   
    }
    
    if(pConfig->bInvertEn)
    {
        pCMP->CR1 |=CMP_CR1_INV_MASK; 
    }    
    else
    {
       pCMP->CR1 &= ~CMP_CR1_INV_MASK;   
    }

    if(pConfig->bHighSpeedEn)
    {
        pCMP->CR1 |=CMP_CR1_PMODE_MASK; 
    }    
    else
    {
       pCMP->CR1 &= ~CMP_CR1_PMODE_MASK;   
    } 

    if(pConfig->bTriggerModeEn)
    {
        pCMP->CR1 |=CMP_CR1_TRIGM_MASK; 
    }    
    else
    {
       pCMP->CR1 &= ~CMP_CR1_TRIGM_MASK;   
    }   
      
    if(pConfig->bWindowEn)
    {
        pCMP->CR1 |=CMP_CR1_WE_MASK; 
    }    
    else
    {
       pCMP->CR1 &= ~CMP_CR1_WE_MASK;   
    } 
    
    if(pConfig->bSampleEn)
    {
        pCMP->CR1 |=CMP_CR1_SE_MASK; 
    }    
    else
    {
       pCMP->CR1 &= ~CMP_CR1_SE_MASK;   
    } 

    pCMP->CR0 = CMP_CR0_FILTER_CNT(pConfig->bFilterCount) | CMP_CR0_HYSTCTR(pConfig->bHysteresisLevel);
    
    if(pConfig->bDMAEn)
    {
        pCMP->SCR |= CMP_SCR_DMAEN_MASK;
    }

    if(pConfig->bDAC6En)
    {
        pCMP->DACCR |= CMP_DACCR_DACEN_MASK;
    } 
    
    if(pConfig->bRefSource)
    {
        pCMP->DACCR |= CMP_DACCR_VRSEL_MASK;
    }
    
    pCMP->DACCR = (pCMP->DACCR & (~CMP_DACCR_VOSEL_MASK)) | CMP_DACCR_VOSEL(pConfig->bDACVoltage);    
    
    pCMP->MUXCR = CMP_MUXCR_PSEL(pConfig->bPlusPin) | CMP_MUXCR_MSEL(pConfig->bMinusPin);
    
    if(pConfig->bCMPEn)
    {
        pCMP->CR1 |=CMP_CR1_EN_MASK; 
    }
}


/*****************************************************************************//*!
*
* @brief deinit CMP module.
*        
* @param   pCMP      pointer to an CMP register base.
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see   CMP_Init.  
*
*****************************************************************************/
void CMP_DeInit(CMP_Type *pCMP) 
{

    
}

/*****************************************************************************//*!
*
* @brief  set up CMP callback routines to be called by interrupt service routine.
*        
* @param  pCMP       pointer to an CMP register base. 
* @param   pfnCallback  callback routine.
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
*****************************************************************************/
void CMP_SetCallback(CMP_Type *pCMP, CMP_CallbackType pfnCallback)
{
    if(CMP0 == pCMP)
    {
        CMP_Callback[0] = pfnCallback;
    }
    else
    {
        CMP_Callback[1] = pfnCallback;
    }
}

/*! @} End of CMP_api_list                                                  */


/*****************************************************************************//*!
*
* @brief  CMP0 interrupt service routine.
*        
* @param  none. 
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
*****************************************************************************/
void CMP0_Isr(void)
{
    if((CMP0->SCR & CMP_SCR_CFF_MASK) == CMP_SCR_CFF_MASK) 
    {
    
        CMP0->SCR |= CMP_SCR_CFF_MASK;    /* clear the falling-edge interrupt flag */
    } 

    if((CMP0->SCR & CMP_SCR_CFR_MASK) == CMP_SCR_CFR_MASK) 
    {
    
        CMP0->SCR |= CMP_SCR_CFR_MASK;    /* clear the rising-edge interrupt flag */
    } 

    if(CMP_Callback[0])
    {
        CMP_Callback[0]();             /* call callback routine */
    }
}

/*****************************************************************************//*!
*
* @brief  CMP1 interrupt service routine.
*        
* @param  none. 
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
*****************************************************************************/
void CMP1_Isr(void)
{
    if((CMP1->SCR & CMP_SCR_CFF_MASK) == CMP_SCR_CFF_MASK) 
    {
        CMP1->SCR |= CMP_SCR_CFF_MASK;    /* clear the falling-edge interrupt flag */
    } 

    if((CMP1->SCR & CMP_SCR_CFR_MASK) == CMP_SCR_CFR_MASK) 
    {
        CMP1->SCR |= CMP_SCR_CFR_MASK;    /* clear the rising-edge interrupt flag */
    } 
    
    if(CMP_Callback[1])
    {
        CMP_Callback[1]();             /* call callback routine */
    }
}













































