/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file CMP.h
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun. 25, 2013
*
* @brief header file for CMP utilities. 
*
*******************************************************************************
*
* provide APIs for accessing CMP
******************************************************************************/
#ifndef _CMP_H_
#define _CMP_H_
#ifdef __cplusplus
extern "C" {
#endif
/******************************************************************************
* Includes
******************************************************************************/

/******************************************************************************
* Constants
******************************************************************************/

/* DAC reference select */
enum
{
    Vin1_VREFH = 0,
    Vin2_VDDA
};

/******************************************************************************
* Macros
******************************************************************************/

/******************************************************************************
 * CMP module number definition                                               *
 ******************************************************************************/
#define     MAX_CMP_NO             2

/******************************************************************************
* CMP positive and negative pin select definition
*
*//*! @addtogroup CMP_pinsel_list
* @{
*******************************************************************************/
#define     CMP0_IN0              0  /*!< CMP0 IN0 input */
#define     CMP0_IN1              1  /*!< CMP0 IN1 input */
#define     DAC12_OUTPUT          3  /*!< CMP0 IN3 input */
#define     CMP0_IN4              4  /*!< CMP0 IN4 input */
#define     CMP0_IN5              5  /*!< CMP0 IN5 input */
#define     BANDGAP               6  /*!< CMP0 IN6 input */
#define     DAC6_OUTPUT           7  /*!< CMP0 IN7 input */

#define     CMP1_IN0              0  /*!< CMP0 IN0 input */
#define     CMP1_IN1              1  /*!< CMP0 IN1 input */
#define     CMP1_IN4              4  /*!< CMP0 IN4 input */
#define     CMP1_IN5              5  /*!< CMP0 IN5 input */



/*! @} End of CMP_pinsel_list                                                */

/******************************************************************************
* CMP interrupt sensitivity edge definition
*
*//*! @addtogroup CMP_intedgesel
* @{
*******************************************************************************/
#define     CMP_FALLING_EDGE_INTERRUPT  1<<3    /*!< interrupt on falling edge  */
#define     CMP_RISING_EDGE_INTERRUPT  1<<4    /*!< interrupt on falling edge  */

/*! @} End of CMP_intedgesel                                                 */


/******************************************************************************
* CMP hysterisis selection definition
*
*//*! @addtogroup CMP_hyst
* @{
*******************************************************************************/

#define     HYST_5MV                  0  /*!< 5mv hysteresis */
#define     HYST_10MV                 1  /*!< 10mv hysteresis*/
#define     HYST_20MV                 2  /*!< 20mv hysteresis */
#define     HYST_30MV                 3  /*!< 30mv hysteresis */
/*! @} End of CMP_hyst                                                       */

/******************************************************************************
* CMP internal DAC reference selection definition
*
*//*! @addtogroup CMP_dacref
* @{
*******************************************************************************/
#define     CMP_DAC6_REF_VREFH     (0<<6)  /*!< select bandgap as 6-bit DAC refference */
#define     CMP_DAC6_REF_VDDA      (1<<6)  /*!< select VDDA as 6-bit DAC refference */
/*! @} End of CMP_dacref                                                     */


/******************************************************************************
* Types
******************************************************************************/

/*! @brief CMP_CALLBACK function declaration                                  */
typedef void (*CMP_CallbackType)(void);
/*! @} End of CMP_callback                                                    */


/******************************************************************************
* CMP config struct
*
*//*! @addtogroup CMP_configstruct
* @{
*******************************************************************************/
/*!
* @brief CMP module configuration struct.
*
*/

typedef struct 
{

    /* CMP0_CR0 register setting bit */
    uint8_t bHysteresisLevel       :2;     /*!< hysteresis control, defines the programmable hysteresis level.  */
    uint8_t bFilterCount           :3 ;     /*!< filter sample count, see reference manual */
    /* CMPx_CR1 register setting bit */
    uint8_t bCMPEn         :1;     /*!< 1: enable CMP, 0:disable CMP  */
    uint8_t bCMPOEn        :1;     /*!< 1: CMPO pin is enabled, 0: disable CMPO out pin  */ 
    uint8_t bCMPOSelect    :1;     /*!< 1: CMPO pin is COUT, 0: CMPO pin is COUNTA  */     
    uint8_t bInvertEn      :1;     /*!< 1: inverts CMP out, 0: do not invert CMP out  */ 
    uint8_t bHighSpeedEn   :1;     /*!< 1: high speed mode, 0: low speed mode */
    uint8_t bTriggerModeEn :1;     /*!< 1: enable trigger, 0: disable trigger */
    uint8_t bWindowEn      :1;     /*!< 1: enable window mode, 0:disable window. */
    uint8_t bSampleEn      :1;     /*!< 1: enable window mode, 0:disable window. */    
    /* CMPx_FPR register setting bit */
    uint8_t u8SamplingPeriod ;     /*!< 1: enable CMP, 0:disable CMP  */
    /* CMPx_SCR register setting bit  */ 
    uint8_t bFallingFlag    :1;     /*!< 1: falling-edge flag has occured, write 1 to is to cleat, 0: falling-edge has not been detected */    
    uint8_t bRisingFlag     :1;     /*!< 1: rising-edge flag has occured, write 1 to is to cleat, 0: rising-edge has not been detected */    
    uint8_t bFallingIntEn   :1;     /*!< 1: falling-edge interrupt is enabled, 0: falling-edge interrupt is disabled  */ 
    uint8_t bRisingIntEn    :1;     /*!< 1: rising-edge interrupt is enabled, 0: rising-edge interrupt is disabled  */   
    uint8_t bDMAEn          :1;     /*!< 1: enable DMA request, 0: disable DMA request  */
    /* CMPx_DACCR register setting bit  */ 
    uint8_t bDACVoltage     :6;     /*!< DAC output voltate select */  
    uint8_t bRefSource      :1;     /*!< 1: reference is from Vin1(VREFH) input, 0: reference is from Vin2(VDD) input   */     
    uint8_t bDAC6En          :1;     /*!< 1: enable 6-bit DAC, 0: disable 6-bit DAC   */     
    /* CMPx_MUXCR register setting bit  */ 
    uint8_t bMinusPin       :3;     /*!< CMP minus input pin select */ 
    uint8_t bPlusPin        :3;     /*!< CMP plus input pin select */     
    
}CMP_ConfigType, *CMP_ConfigPtr;
/*! @} End of CMP_configstruct                                               */

/******************************************************************************
* Global variables
******************************************************************************/

/*!
 * inline functions
 */
/******************************************************************************
* CMP api list.
*
*//*! @addtogroup CMP_api_list
* @{
*******************************************************************************/
/*****************************************************************************//*!
*
* @brief  enable the CMP module.
*        
* @param[in]  pCMP      pointer to an CMP module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    CMP_Disable.
*
*****************************************************************************/
__STATIC_INLINE void CMP_Enable(CMP_Type *pCMP)
{
    pCMP->CR1 |= CMP_CR1_EN_MASK ;
}

/*****************************************************************************//*!
*
* @brief  disable the CMP module.
*        
* @param[in]  pCMP      pointer to an CMP module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    CMP_Enable.
*
*****************************************************************************/
__STATIC_INLINE void CMP_Disable(CMP_Type *pCMP)
{
    pCMP->CR1 &= ~CMP_CR1_EN_MASK ;
}

/*****************************************************************************//*!
*
* @brief  falling-edge interrupt enable.
*        
* @param[in]  pCMP         pointer to an CMP module.
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
*****************************************************************************/
__STATIC_INLINE void CMP_FallingInterruptEnable(CMP_Type *pCMP)
{

    pCMP->SCR |= CMP_SCR_IEF_MASK;
}

/*****************************************************************************//*!
*
* @brief  falling-edge interrupt enable.
*        
* @param[in]  pCMP         pointer to an CMP module.
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
*****************************************************************************/
__STATIC_INLINE void CMP_FallingInterruptDisable(CMP_Type *pCMP)
{

    pCMP->SCR &= CMP_SCR_IEF_MASK;
}

/*****************************************************************************//*!
*
* @brief  rising-edge interrupt enable.
*        
* @param[in]  pCMP         pointer to an CMP module.
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
*****************************************************************************/
__STATIC_INLINE void CMP_RisingInterruptEnable(CMP_Type *pCMP)
{

    pCMP->SCR |= CMP_SCR_IER_MASK;
}

/*****************************************************************************//*!
*
* @brief  falling-edge interrupt enable.
*        
* @param[in]  pCMP         pointer to an CMP module.
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
*****************************************************************************/
__STATIC_INLINE void CMP_RisingInterruptDisable(CMP_Type *pCMP)
{

    pCMP->SCR &= CMP_SCR_IER_MASK;
}

/*****************************************************************************//*!
*
* @brief  enable the CMP module analog comparator output.
*        
* @param[in]  pCMP      pointer to an CMP module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    CMP_DisablePinOut.
*
*****************************************************************************/
__STATIC_INLINE void CMP_OutputEnable(CMP_Type *pCMP)
{
    pCMP->CR1 |= CMP_CR1_OPE_MASK;
}

/*****************************************************************************//*!
*
* @brief  disable the CMP module analog comparator output.
*        
* @param[in]  pCMP      pointer to an CMP module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    CMP_EnablePinOut.
*
*****************************************************************************/
__STATIC_INLINE void CMP_OutputDisable(CMP_Type *pCMP)
{
    pCMP->CR1 &= ~CMP_CR1_OPE_MASK;
}

/*****************************************************************************//*!
*
* @brief  select CMP hystersis. 
*        
* @param[in]  pCMP           pointer to an CMP module.
* @param[in]  u8HystSelect     CMP_HYST_20MV or CMP_HYST_30MV.   
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
*****************************************************************************/
__STATIC_INLINE void CMP_SelectHyst(CMP_Type *pCMP, uint8_t u8HystSelect)
{
    pCMP->CR0 &= ~CMP_CR0_HYSTCTR_MASK;
    pCMP->CR0 |= u8HystSelect;
}

/*****************************************************************************//*!
*
* @brief  get the falling-edge interrupt flag bit.
*        
* @param[in]  pCMP      pointer to an CMP module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    CMP_ClrFlag.
*
*****************************************************************************/
__STATIC_INLINE uint8_t CMP_GetFallingFlag(CMP_Type *pCMP)
{
    return (pCMP->SCR & CMP_SCR_CFF_MASK);
}

/*****************************************************************************//*!
*
* @brief  clear the falling-edge interrupt flag bit.
*        
* @param[in]  pCMP      pointer to an CMP module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    CMP_GetFlag.
*
*****************************************************************************/
__STATIC_INLINE void CMP_ClrFallingFlag(CMP_Type *pCMP)
{
    pCMP->SCR |= CMP_SCR_CFF_MASK;
}

/*****************************************************************************//*!
*
* @brief  get the rising-edge interrupt flag bit.
*        
* @param[in]  pCMP      pointer to an CMP module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    CMP_ClrFlag.
*
*****************************************************************************/
__STATIC_INLINE uint8_t CMP_GetRisingFlag(CMP_Type *pCMP)
{
    return (pCMP->SCR & CMP_SCR_CFR_MASK);
}

/*****************************************************************************//*!
*
* @brief  clear the rising-edge interrupt flag bit.
*        
* @param[in]  pCMP      pointer to an CMP module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    CMP_GetFlag.
*
*****************************************************************************/
__STATIC_INLINE void CMP_ClrRisingFlag(CMP_Type *pCMP)
{
    pCMP->SCR |= CMP_SCR_CFR_MASK;
}

/*****************************************************************************//*!
*
* @brief  CMP Plus Input Select.
*        
* @param[in]  pCMP        pointer to an CMP module.
* @param[in]  u8PlusPinSelect   plus input select     
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    CMP_PlusInputSelect.
*
*****************************************************************************/
__STATIC_INLINE void CMP_PlusInputSelect(CMP_Type *pCMP, uint8_t u8PlusPinSelect)
{
    pCMP->MUXCR &= ~CMP_MUXCR_PSEL_MASK;
    pCMP->MUXCR |= CMP_MUXCR_PSEL(u8PlusPinSelect);
}

/*****************************************************************************//*!
*
* @brief  CMP Minus Input Select.
*        
* @param[in]  pCMP        pointer to an CMP module.
* @param[in]  u8MinusPinSelect   minus input select  
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    u8MinusPinSelect.
*
*****************************************************************************/
__STATIC_INLINE void CMP_MinusInputSelect(CMP_Type *pCMP, uint8_t u8MinusPinSelect)
{
    pCMP->MUXCR &= ~~CMP_MUXCR_MSEL_MASK;
    pCMP->MUXCR|= u8MinusPinSelect;
}

/*****************************************************************************//*!
*
* @brief  Enable 6 bit DAC in CMP module.
*        
* @param[in]  pCMP      pointer to an CMP module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    CMP_DacDisable.
*
*****************************************************************************/
__STATIC_INLINE void CMP_Dac6Enable(CMP_Type *pCMP)
{
    pCMP->DACCR |= CMP_DACCR_DACEN_MASK;
}

/*****************************************************************************//*!
*
* @brief  Disable 6 bit DAC in CMP module.
*        
* @param[in]  pCMP      pointer to an CMP module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    CMP_DacEnable.
*
*****************************************************************************/
__STATIC_INLINE void CMP_DacDisable(CMP_Type *pCMP)
{
    pCMP->DACCR &= ~CMP_DACCR_DACEN_MASK;
}

/*****************************************************************************//*!
*
* @brief  Set Vin1 as CMP 6 bit DAC Reference.
*        
* @param[in]  pCMP        pointer to an CMP module.
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
*****************************************************************************/
__STATIC_INLINE void CMP_SetVin1AsRef(CMP_Type *pCMP)
{
    pCMP->DACCR &= ~CMP_DACCR_VRSEL_MASK;
}

/*****************************************************************************//*!
*
* @brief  Set Vin2 as CMP 6 bit DAC Reference.
*        
* @param[in]  pCMP        pointer to an CMP module.
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
*****************************************************************************/
__STATIC_INLINE void CMP_SetVin2sRef(CMP_Type *pCMP)
{
    pCMP->DACCR |= CMP_DACCR_VRSEL_MASK;
}

/*****************************************************************************//*!
*
* @brief  CMP 6 bit DAC Output Value Set.
*        
* @param[in]  pCMP        pointer to an CMP module.
* @param[in]  u8DacValue    dac output set, Voutput= (Vin/64)x(DACVAL[5:0]+1).
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
*****************************************************************************/
__STATIC_INLINE void CMP_DacOutputSet(CMP_Type *pCMP, uint8_t u8DacValue)
{
    ASSERT(!(u8DacValue & (~CMP_DACCR_VOSEL_MASK)));
    pCMP->DACCR &= ~CMP_DACCR_VOSEL_MASK;
    pCMP->DACCR |= CMP_DACCR_VOSEL(u8DacValue);
}


/******************************************************************************
* Global functions
******************************************************************************/
void CMP_Init(CMP_Type *pCMP, CMP_ConfigType *pConfig);
void CMP_DeInit(CMP_Type *pCMP); 
void CMP_SetCallback(CMP_Type *pCMP, CMP_CallbackType pfnCallback);
/*! @} End of CMP_api_list                                                  */
#ifdef __cplusplus
}
#endif
#endif /* _CMP_H_ */












































































































































