
/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file dac.c
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun. 25, 2013
*
* @brief providing APIs for configuring DAC. 
*
*******************************************************************************
*
* provide APIs for configuring DAC
******************************************************************************/
#include "common.h"
#include "dac.h"
/******************************************************************************
* Global variables
******************************************************************************/

/******************************************************************************
* Constants and macros
******************************************************************************/

/******************************************************************************
* Local types
******************************************************************************/

/******************************************************************************
* Local function prototypes
******************************************************************************/

/******************************************************************************
* Local variables
******************************************************************************/

DAC_CallbackType DAC_Callback[1] = {NULL};

/******************************************************************************
* Local functions
******************************************************************************/
/******************************************************************************
* Global functions
******************************************************************************/
void DAC0_Isr(void);

/******************************************************************************
* DAC api list.
*
*//*! @addtogroup DAC_api_list
* @{
*******************************************************************************/

/*****************************************************************************//*!
*
* @brief initialize DAC as per control field.
*        
* @param   pDAC         pointer to an DAC register base.
* @param   pConfig        control parameters.  
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see   DAC_DeInit.  
*
*****************************************************************************/
void DAC_Init(DAC_Type *pDAC, DAC_ConfigType *pConfig) 
{        
    /* enable DAC clock gate */
    SIM->SCGC6 |= SIM_SCGC6_DAC0_MASK;
    
      if(pConfig->bDACEn)
    {
        pDAC->C0 |=DAC_C0_DACEN_MASK; 
    }      
    if(pConfig->bBufferBottomIntEn || pConfig->bBufferTopIntEn ||pConfig->bWatermarkIntEn  ) 
    {  
        NVIC_EnableIRQ( DAC0_IRQn );
    }
    
    if(pConfig->bBufferBottomIntEn ) 
    {
        pDAC->C0 |= DAC_C0_DACBBIEN_MASK;
    } 
   
    if(pConfig->bBufferTopIntEn) 
    {
        pDAC->C0 |= DAC_C0_DACBTIEN_MASK;
    }
  
    if(pConfig->bWatermarkIntEn) 
    {
        pDAC->C0 |= DAC_C0_DACBWIEN_MASK;
    }

    if(pConfig->bWatermarkIntEn) 
    {
        pDAC->C0 |= DAC_C0_DACBWIEN_MASK;
    }

    if(pConfig->bLowPowerEn) 
    {
        pDAC->C0 |= DAC_C0_LPEN_MASK;
    }
    
    if(pConfig->bSoftwareTriggerEn) 
    {
        pDAC->C0 |= DAC_C0_DACTRGSEL_MASK;
    }
    
    if(pConfig->bDACRefSelect) 
    {
        pDAC->C0 |= DAC_C0_DACRFS_MASK;
    }

    if(pConfig->bBufferEn)
    {
        pDAC->C1 |=DAC_C1_DACBFEN_MASK; 
    }
    
    pDAC->C1 = (pDAC->C1 & (~DAC_C1_DACBFMD_MASK)) | DAC_C1_DACBFMD(pConfig->bBufferModeSelect);
    
    pDAC->C1 = (pDAC->C1 & (~DAC_C1_DACBFWM_MASK)) | DAC_C1_DACBFWM(pConfig->bWatermarkSelect);
    
    if(pConfig->bBufferUpperLimit)
    {
        pDAC->C2 |=DAC_C2_DACBFUP_MASK; 
    }                

    if(pConfig->bBufferReadPointer)
    {
        pDAC->C2 |=DAC_C2_DACBFRP_MASK; 
    } 
    
        

        
     //DAC0->DAT[0].DATL = pConfig->Data0Low; 
    // DAC0->DAT[0].DATH = pConfig->Data0High;
     
     //DAC0->DAT[1].DATL = pConfig->Data1Low; 
     //DAC0->DAT[1].DATH = pConfig->Data1High;     
}


/*****************************************************************************//*!
*
* @brief deinit DAC module.
*        
* @param   pDAC      pointer to an DAC register base.
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see   DAC_Init.  
*
*****************************************************************************/
void DAC_DeInit(DAC_Type *pDAC) 
{

    
}

/*****************************************************************************//*!
*
* @brief  set up DAC callback routines to be called by interrupt service routine.
*        
* @param  pDAC       pointer to an DAC register base. 
* @param   pfnCallback  callback routine.
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
*****************************************************************************/
void DAC_SetCallback(DAC_Type *pDAC, DAC_CallbackType pfnCallback)
{
    DAC_Callback[0] = pfnCallback;
}

/*! @} End of DAC_api_list                                                  */


/*****************************************************************************//*!
*
* @brief  DAC0 interrupt service routine.
*        
* @param  none. 
*
* @return none.
*
* @ Pass/ Fail criteria: none.
*
*****************************************************************************/
void DAC0_Isr(void)
{
    if(DAC_Callback[0])
    {
        DAC_Callback[0]();             /* call callback routine */
    }
}

