/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file dma.h
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun. 25, 2013
*
* @brief header file for eDMA module utilities (eDMA). 
*
*******************************************************************************
*
* provide APIs for accessing eDMA module (eDMA)
******************************************************************************/
#ifndef _DMA_H__
#define _DMA_H__
#ifdef __cplusplus
extern "C" {
#endif
/******************************************************************************
* Global variables
******************************************************************************/

/******************************************************************************
* Constants and macros
******************************************************************************/
/******************************************************************************
*
*//*! @addtogroup dma_request_source_list
* @{
*******************************************************************************/
/*!
 * @brief DAM request source
 *
 */
#define DMA_SLOTS_UART0_RX              2
#define DMA_SLOTS_UART0_TX              3
#define DMA_SLOTS_UART1_RX              4
#define DMA_SLOTS_UART1_TX              5    
#define DMA_SLOTS_SPI0_RX               16
#define DMA_SLOTS_SPI0_TX               17
#define DMA_SLOTS_I2C0_RX_TX            22
#define DMA_SLOTS_FTM0_CH0              24  
#define DMA_SLOTS_FTM0_CH1              25  
#define DMA_SLOTS_FTM0_CH2              26  
#define DMA_SLOTS_FTM0_CH3              27  
#define DMA_SLOTS_FTM0_CH4              28  
#define DMA_SLOTS_FTM0_CH5              29   
#define DMA_SLOTS_FTM1_CH0              32  
#define DMA_SLOTS_FTM1_CH1              33  
#define DMA_SLOTS_FTM2_CH0              34  
#define DMA_SLOTS_FTM2_CH1              35   
#define DMA_SLOTS_ADC0                  40  
#define DMA_SLOTS_ADC1                  41  
#define DMA_SLOTS_CMP0                  42  
#define DMA_SLOTS_CMP1                  43
#define DMA_SLOTS_DAC0                  45   
#define DMA_SLOTS_PDB                   48
#define DMA_SLOTS_PORT_A                49    
#define DMA_SLOTS_PORT_B                50   
#define DMA_SLOTS_PORT_C                51   
#define DMA_SLOTS_PORT_D                52
#define DMA_SLOTS_PORT_E                53
#define DMA_SLOTS_ALWAYS_EN             54
/*! @} End of dma_request_source_list   */    
/******************************************************************************
*
*//*! @addtogroup dma_channel_arbitration_selection_enum
* @{
*******************************************************************************/
/*!
 * @brief DAM channel arbitration selection
 *
 */
typedef enum{
Fixed_Priority_Arbitration = 0,
Round_Robin_Arbitration = 1,
}DMA_ChannelArbType;
/*! @} End of dma_channel_arbitration_selection_enum    */

/******************************************************************************
*
*//*! @addtogroup dma_error_status_list
* @{
*******************************************************************************/
/*!
 * @brief DAM Error Status Register (DMA_ES)
 *
 */
typedef enum{
ES_VLD_MASK = 0x01<<31,  /*!< Logical OR of all ERR status bits */
ES_ECX_MASK = 0x01<<16,  /*!< Transfer Canceled */
ES_CPE_MASK = 0x01<<14,  /*!< Channel Priority Error */
ES_ERRCHN_MASK = 0x03<<8,/*!< Error Channel Number or Canceled Channel Number */
ES_SAE_MASK = 0x01<<7,	 /*!< Source Address Error */
ES_SOE_MASK = 0x01<<6,	 /*!< Source Offset Error */
ES_DAE_MASK = 0x01<<5,	 /*!< Destination Address Error */
ES_DOE_MASK = 0x01<<4,	 /*!< Destination Offset Error */
ES_NCE_MASK = 0x01<<3,	 /*!< NBYTES/CITER Configuration Error */
ES_SGE_MASK = 0x01<<2,   /*!< Scatter/Gather Configuration Error */
ES_SBE_MASK = 0x01<<1,   /*!< Source Bus Error */
ES_DBE_MASK = 0x01<<0,   /*!< Destination Bus Error */
}DMA_ErrorStatusType;
/*! @} End of dma_error_status_list    */



/******************************************************************************
*
*//*! @addtogroup dma_tcd_type
* @{
*******************************************************************************/
/*!
 * @brief TCD type.
 *
 */
typedef struct {	
	uint32_t SADDR; 
	uint16_t SOFF; 							 /*!< TCD Signed Source Address Offset, array offset: 0x1004, array step: 0x20 */
	uint16_t ATTR; 							 /*!< TCD Transfer Attributes, array offset: 0x1006, array step: 0x20 */
	union { 								 /*!< offset: 0x1008, array step: 0x20 */
	  uint32_t NBYTES_MLNO;					 /*!< TCD Minor Byte Count (Minor Loop Disabled), array offset: 0x1008, array step: 0x20 */
	  uint32_t NBYTES_MLOFFNO; 				 /*!< TCD Signed Minor Loop Offset (Minor Loop Enabled and Offset Disabled), array offset: 0x1008, array step: 0x20 */
	  uint32_t NBYTES_MLOFFYES;				 /*!< TCD Signed Minor Loop Offset (Minor Loop and Offset Enabled), array offset: 0x1008, array step: 0x20 */
	};
	uint32_t SLAST;							 /*!< TCD Last Source Address Adjustment, array offset: 0x100C, array step: 0x20 */
	uint32_t DADDR;							 /*!< TCD Destination Address, array offset: 0x1010, array step: 0x20 */
	uint16_t DOFF; 							 /*!< TCD Signed Destination Address Offset, array offset: 0x1014, array step: 0x20 */
	union { 								 /*!< offset: 0x1016, array step: 0x20 */
	  uint16_t CITER_ELINKNO;				 /*!< TCD Current Minor Loop Link, Major Loop Count (Channel Linking Disabled), array offset: 0x1016, array step: 0x20 */
	  uint16_t CITER_ELINKYES; 				 /*!< TCD Current Minor Loop Link, Major Loop Count (Channel Linking Enabled), array offset: 0x1016, array step: 0x20 */
	};
	uint32_t DLAST_SGA;						 /*!< TCD Last Destination Address Adjustment/Scatter Gather Address, array offset: 0x1018, array step: 0x20 */
	uint16_t CSR;							 /*!< TCD Control and Status, array offset: 0x101C, array step: 0x20 */
	union { 								 /*!< offset: 0x101E, array step: 0x20 */
	  uint16_t BITER_ELINKNO;				 /*!< TCD Beginning Minor Loop Link, Major Loop Count (Channel Linking Disabled), array offset: 0x101E, array step: 0x20 */
	  uint16_t BITER_ELINKYES; 				 /*!< TCD Beginning Minor Loop Link, Major Loop Count (Channel Linking Enabled), array offset: 0x101E, array step: 0x20 */
		};
} DMA_TCDType;

/*! @} End of dma_tcd_type      					*/

/******************************************************************************
*
*//*! @addtogroup dma_setting_type
* @{
*******************************************************************************/
/*!
 * @brief DMA setting type.
 *
 */
typedef struct {							
	uint16_t bHOE;				/*!<  Halt On Error    */
	uint16_t bRoundRobinEn;		/*!<  Enable Round Robin Channel Arbitration    */
	uint16_t bDebugEn;			/*!<  Enable Debug  */
	uint16_t bErrorIntEn;		/*!<  Error Interrupt Enable  */
	uint16_t bAsysReqEn;        /*!<  Asyschronous request enable  */
	uint16_t bIntEn;            /*!<  transferring complete interrupt enable  */
    uint16_t bReqEn;            /*!<  request enable  */
} DMA_SettingType;

/*! @} End of dma_config_type     */


/******************************************************************************
*
*//*! @addtogroup dma_config_type
* @{
*******************************************************************************/
/*!
 * @brief DMA config type.
 *
 */
typedef struct {							 
	DMA_SettingType sSetting;   /*!<  DMA setting    */
	DMA_TCDType sTCDInfo;       /*!<  TCD description information */
	uint8_t Channel;            /*!<  Channel number */
	uint8_t Slot;               /*!<  DMA request source */
} DMA_ConfigType;

/*! @} End of dma_config_type     */

/******************************************************************************
*
*//*! @addtogroup dma_callback
* @{
*******************************************************************************/
typedef void (*DMA_CallbackType)(void);		/*!< DMA call back function */
/*! @} End of dma_callback                                             						*/

/******************************************************************************
* inline functions
******************************************************************************/
/******************************************************************************
*
*//*! @addtogroup dma_api_list
* @{
*******************************************************************************/
/*****************************************************************************//*!
   *
   * @brief Cancel the remaining data transfer.
   *        
   * @param[in] pDMAx      point to DMA module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_CancelTransfer(DMA_Type *pDMAx)
{
    pDMAx->CR |= DMA_CR_CX_MASK;
}
/*****************************************************************************//*!
   *
   * @brief Cancel the remaining data transfer and updated the error status register.
   *        
   * @param[in] pDMAx      point to DMA module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_ErrorCancelTransfer(DMA_Type *pDMAx)
{
    pDMAx->CR |= DMA_CR_ECX_MASK;
}
/*****************************************************************************//*!
   *
   * @brief Enable Minor Loop Mapping.
   *        
   * @param[in] pDMAx      point to DMA module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_MinorLoopEn(DMA_Type *pDMAx)
{
    pDMAx->CR |= DMA_CR_EMLM_MASK;
}
/*****************************************************************************//*!
   *
   * @brief Disable Minor Loop Mapping.
   *        
   * @param[in] pDMAx      point to DMA module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_MinorLoopDis(DMA_Type *pDMAx)
{
    pDMAx->CR &= ~DMA_CR_EMLM_MASK;
}

/*****************************************************************************//*!
   *
   * @brief Continuous Link Mode Enable.
   *        
   * @param[in] pDMAx      point to DMA module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_ContinuousLinkModeEn(DMA_Type *pDMAx)
{
    pDMAx->CR |= DMA_CR_CLM_MASK;
}
/*****************************************************************************//*!
   *
   * @brief Continuous Link Mode Disable.
   *        
   * @param[in] pDMAx      point to DMA module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_ContinuousLinkModeDis(DMA_Type *pDMAx)
{
    pDMAx->CR &= ~DMA_CR_CLM_MASK;
}
/*****************************************************************************//*!
   *
   * @brief Halt DMA Operations.
   *        
   * @param[in] pDMAx      point to DMA module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_HaltDMAOperation(DMA_Type *pDMAx)
{
    pDMAx->CR |= DMA_CR_HALT_MASK;
}
/*****************************************************************************//*!
   *
   * @brief Halt DMA Operations on errors.
   *        
   * @param[in] pDMAx      point to DMA module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_HaltDMAOnError(DMA_Type *pDMAx)
{
    pDMAx->CR |= DMA_CR_HOE_MASK;
}
/*****************************************************************************//*!
   *
   * @brief Not Halt DMA Operations on errors.
   *        
   * @param[in] pDMAx      point to DMA module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_NoHaltDMAOnError(DMA_Type *pDMAx)
{
    pDMAx->CR &= ~DMA_CR_HOE_MASK;
}
/*****************************************************************************//*!
   *
   * @brief Channel Arbitration Selection.
   *        
   * @param[in] pDMAx      point to DMA module type.
   * @param[in] bArb         DMA arbitration type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_ChannelArbSelection(DMA_Type *pDMAx,DMA_ChannelArbType bArb)
{
    if(bArb)
    {
		pDMAx->CR |= DMA_CR_ERCA_MASK;
    }
    else
    {
		pDMAx->CR &= ~DMA_CR_ERCA_MASK;
    }
}
/*****************************************************************************//*!
   *
   * @brief DMA Debug Enable.
   *        
   * @param[in] pDMAx      point to DMA module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_DebugEn(DMA_Type *pDMAx)
{
    pDMAx->CR |= DMA_CR_EDBG_MASK;
}
/*****************************************************************************//*!
   *
   * @brief DMA Debug Disable.
   *        
   * @param[in] pDMAx      point to DMA module type.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_DebugDis(DMA_Type *pDMAx)
{
    pDMAx->CR &= ~DMA_CR_EDBG_MASK;
}

/*****************************************************************************//*!
   *
   * @brief DMA Debug Disable.
   *        
   * @param[in] pDMAx      point to DMA module type.
   * @param[in] ErrorStatusMask      Mask of error status.
   *
   * @return error status
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE uint32_t DMA_CheckErrorStatus(DMA_Type *pDMAx,DMA_ErrorStatusType ErrorStatusMask)
{
   return (pDMAx->ES & ErrorStatusMask);
}
/*****************************************************************************//*!
   *
   * @brief DMA write ERQ register.
   *        
   * @param[in] pDMAx      point to DMA module type.
   * @param[in] ERQn        enable DMA request.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_WriteERQReg(DMA_Type *pDMAx,uint32_t ERQn)
{
    pDMAx->ERQ = (ERQn&0x0f);
}
/*****************************************************************************//*!
   *
   * @brief DMA Set Enable Error Interrupt Register.
   *        
   * @param[in] pDMAx      point to DMA module type.
   * @param[in] EEIn        the channel number to enable error interrupt, if bit 6 is 1, all channel is enable.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_SetEEIReg(DMA_Type *pDMAx,uint8_t EEIn)
{
    pDMAx->SEEI = EEIn;
}
/*****************************************************************************//*!
   *
   * @brief DMA Clear Enable Error Interrupt Register.
   *        
   * @param[in] pDMAx    point to DMA module type.
   * @param[in] EEIn       the channel number to disable error interrupt, if bit 6 is 1, all channel is disable.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_ClearEEIReg(DMA_Type *pDMAx,uint8_t EEIn)
{
    pDMAx->CEEI = EEIn;
}
/*****************************************************************************//*!
   *
   * @brief DMA Set Enable Requests Register.
   *        
   * @param[in] pDMAx      point to DMA module type.
   * @param[in] ERQn        the channel number to enable request, if bit 6 is 1, all channel request is enable.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_SetERQReg(DMA_Type *pDMAx,uint8_t ERQn)
{
    pDMAx->SERQ = ERQn;
}
/*****************************************************************************//*!
   *
   * @brief DMA Clear Enable Error Interrupt Register.
   *        
   * @param[in] pDMAx    point to DMA module type.
   * @param[in] ERQn       the channel number to disable error interrupt, if bit 6 is 1, all channel request is disable.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_ClearERQReg(DMA_Type *pDMAx,uint8_t ERQn)
{
    pDMAx->CERQ = ERQn;
}
/*****************************************************************************//*!
   *
   * @brief DMA Clear DONE status bit.
   *        
   * @param[in] pDMAx    point to DMA module type.
   * @param[in] CDNEn    Clears the corresponding bit in TCDn_CSR[DONE], if bit 6 is 1, clear all bit.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_ClearDONEbit(DMA_Type *pDMAx,uint8_t CDNEn)
{
    pDMAx->CDNE = CDNEn;
}
/*****************************************************************************//*!
   *
   * @brief DMA Set START status bit.
   *        
   * @param[in] pDMAx    point to DMA module type.
   * @param[in] SSRTn    Set the corresponding bit in TCDn_CSR[START], if bit 6 is 1, set all bit.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_SetSTARTbit(DMA_Type *pDMAx,uint8_t SSRTn)
{
    pDMAx->SSRT = SSRTn;
}
/*****************************************************************************//*!
   *
   * @brief DMA Clear error bit.
   *        
   * @param[in] pDMAx    point to DMA module type.
   * @param[in] ERRn    Clear the corresponding bit in ERR, if bit 6 is 1, clear all bit.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_ClearERRbit(DMA_Type *pDMAx,uint8_t ERRn)
{
    pDMAx->CERR = ERRn;
}
/*****************************************************************************//*!
   *
   * @brief clear interrupt request register.
   *        
   * @param[in] pDMAx    point to DMA module type.
   * @param[in] INTn      Clear the corresponding bit in INT, if bit 6 is 1, clear all bit.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_ClearINTReg(DMA_Type *pDMAx,uint8_t INTn)
{
    pDMAx->CINT = INTn;
}

/*****************************************************************************//*!
   *
   * @brief write interrupt request register.
   *        
   * @param[in] pDMAx    point to DMA module type.
   * @param[in] INTn      the value write to INT register.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_WriteINTReg(DMA_Type *pDMAx,uint8_t INTn)
{
    pDMAx->INT = INTn;
}
/*****************************************************************************//*!
   *
   * @brief read interrupt request register.
   *        
   * @param[in] pDMAx    point to DMA module type.
   *
   * @return INT register
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE uint8_t DMA_ReadINTReg(DMA_Type *pDMAx)
{
    return pDMAx->INT;
}
/*****************************************************************************//*!
   *
   * @brief read Error register.
   *        
   * @param[in] pDMAx    point to DMA module type.
   *
   * @return Error register
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE uint8_t DMA_ReadERRReg(DMA_Type *pDMAx)
{
    return pDMAx->ERR;
}
/*****************************************************************************//*!
   *
   * @brief read Hardware Request Status Register.
   *        
   * @param[in] pDMAx    point to DMA module type.
   *
   * @return Hardware Request Status Register
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE uint8_t DMA_ReadHRSReg(DMA_Type *pDMAx)
{
    return pDMAx->HRS;
}
/*****************************************************************************//*!
   *
   * @brief Enable Asynchronous Request in Stop Register.
   *        
   * @param[in] pDMAx    point to DMA module type.
   * @param[in] slots      DMA channel source
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_ChannelEn(DMA_Type *pDMAx,uint8_t Channel,uint8_t slots)
{
    ASSERT((slots<63)||(Channel));
    SIM->SCGC6 |= SIM_SCGC6_DMAMUX_MASK;
    SIM->SCGC7 |= SIM_SCGC7_DMA_MASK;
    DMAMUX->CHCFG[Channel] = DMAMUX_CHCFG_SOURCE(slots)|DMAMUX_CHCFG_ENBL_MASK;
}

/*****************************************************************************//*!
   *
   * @brief Enable Asynchronous Request in Stop Register.
   *        
   * @param[in] pDMAx    point to DMA module type.
   * @param[in] Chn      Enable asynchronous DMA request to corresponding channel.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void DMA_AsynReqEn(DMA_Type *pDMAx,uint8_t Chn)
{
    pDMAx->EARS = Chn;
}


/******************************************************************************
* Global functions
******************************************************************************/
void DMA_CancelTransfer(DMA_Type *pDMAx);
void DMA_ErrorCancelTransfer(DMA_Type *pDMAx);
void DMA_MinorLoopEn(DMA_Type *pDMAx);
void DMA_MinorLoopDis(DMA_Type *pDMAx);
void DMA_ContinuousLinkModeEn(DMA_Type *pDMAx);
void DMA_ContinuousLinkModeDis(DMA_Type *pDMAx);
void DMA_HaltDMAOperation(DMA_Type *pDMAx);
void DMA_HaltDMAOnError(DMA_Type *pDMAx);
void DMA_NoHaltDMAOnError(DMA_Type *pDMAx);
void DMA_ChannelArbSelection(DMA_Type *pDMAx,DMA_ChannelArbType bArb);
void DMA_DebugEn(DMA_Type *pDMAx);
void DMA_DebugDis(DMA_Type *pDMAx);
uint32_t DMA_CheckErrorStatus(DMA_Type *pDMAx,DMA_ErrorStatusType ErrorStatusMask);
void DMA_WriteERQReg(DMA_Type *pDMAx,uint32_t ERQn);
void DMA_SetEEIReg(DMA_Type *pDMAx,uint8_t EEIn);
void DMA_ClearEEIReg(DMA_Type *pDMAx,uint8_t EEIn);
void DMA_SetERQReg(DMA_Type *pDMAx,uint8_t ERQn);
void DMA_ClearEERQReg(DMA_Type *pDMAx,uint8_t ERQn);
void DMA_ClearDONEbit(DMA_Type *pDMAx,uint8_t CDNEn);
void DMA_SetSTARTbit(DMA_Type *pDMAx,uint8_t SSRTn);
void DMA_ClearERRbit(DMA_Type *pDMAx,uint8_t ERRn);
void DMA_ClearINTReg(DMA_Type *pDMAx,uint8_t INTn);
void DMA_WriteINTReg(DMA_Type *pDMAx,uint8_t INTn);
uint8_t DMA_ReadINTReg(DMA_Type *pDMAx);
uint8_t DMA_ReadERRReg(DMA_Type *pDMAx);
uint8_t DMA_ReadHRSReg(DMA_Type *pDMAx);
void DMA_AsynReqEn(DMA_Type *pDMAx,uint8_t Chn);
void DMA_ChannelEn(DMA_Type *pDMAx,uint8_t Channel,uint8_t slots);

/*! @} End of i2c_bus_state_list     */

void DMA_Init(DMA_Type *pDMAx,DMA_ConfigType *pDMAConfig);
void DMA_UpdatedTCDInfo(DMA_Type *pDMAx,uint8_t Channel,DMA_TCDType *pTCD);
void DMA_SetChannelPriority(DMA_Type *pDMAx,uint8_t Channel,uint8_t Priority);
void DMA_SetCallBack( DMA_CallbackType pCallBack,uint8_t Channel );



#ifdef __cplusplus
}
#endif
#endif //


