/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file     lptmr.c
*
* @author   Freescale
*
* @version  0.0.1
*
* @date     Jun 25, 2013
*
* @brief    Programmable Delay Block (lptmr) source code. 
*
******************************************************************************/
#include "common.h"
#include "lptmr.h"

/******************************************************************************
* Global variables
******************************************************************************/

/******************************************************************************
* Constants and macros
******************************************************************************/

/******************************************************************************
* Local types
******************************************************************************/

/******************************************************************************
* Local function prototypes
******************************************************************************/

/******************************************************************************
* Local variables
******************************************************************************/
/*!
 * @brief global variable to store LPTMR callbacks.
 *
 */
LPTMR_CallbackType LPTMR_Callback[2] = {(LPTMR_CallbackType)NULL}; /*!< lptmr initial callback */

/******************************************************************************
* Local functions
******************************************************************************/
void LPTMR_Ch0Isr(void);
void LPTMR_Ch1Isr(void);


/******************************************************************************
* Global functions
******************************************************************************/

/******************************************************************************
* define lptmr APIs
*
*//*! @addtogroup lptmr_api_list
* @{
*******************************************************************************/

/*****************************************************************************//*!
*
* @brief initialize lptmr module.
*        
* @param[in]   pConfig point to configuration  
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void LPTMR_Init(LPTMR_Type *pLPTMR, LPTMR_ConfigType *pConfig)
{  
    SIM->SCGC5 |= SIM_SCGC5_LPTMR_MASK;         /*!< enable lptmr clock gate */
           
    if (pConfig->bInterruptEn )            
    {                                                 
        LPTMR_IntEnable(LPTMR0);
        NVIC_EnableIRQ(LPTMR0_IRQn);          /*!< enable lptmr vector */                   
    }                   
    else                                     
    {                     
        NVIC_DisableIRQ(LPTMR0_IRQn);         /*!< disable lptmr vector */        
    }
    
    if (pConfig->bTimerMode )            
    {  
        LPTMR_SetPulseMode(LPTMR0);
    }      
    
    if (pConfig->bFreeRuning )            
    {  
        LPTMR_SetFreeRunning(LPTMR0);
    } 

    if (pConfig->bPinPolarity )            
    {  
        LPTMR_SetPinHigh(LPTMR0);
    } 
    
    if (pConfig->bPrescaleBypassEn )            
    {  
        LPTMR_PrescaleBypass(LPTMR0);
    }    
    
    LPTMR_PrescaleSelect(LPTMR0, pConfig->bPrescaleSelect);
    
    LPTMR_ClockSelect(LPTMR0, pConfig->bClockSelect);

    if (pConfig->bLPTMREn)            
    {                                               
        LPTMR_Enable(LPTMR0);                       /*!< enable lptmr module */           
    } 

}

/*****************************************************************************//*!
*
* @brief lptmr module set call back.
*          
* @param[in] pfnCallback point to call back.   
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void LPTMR_SetCallback(LPTMR_CallbackType pfnCallback)
{
  LPTMR_Callback[0] = pfnCallback;
}


/*! @} End of lptmr_api_list                                                   */


/*****************************************************************************//*!
*
* @brief lptmr module isr.
*        
* @param none
*
* @return none 
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void LPTMR_Isr(void) 
{   
   //LPTMR_ClrFlags(0);  
    
    if (LPTMR_Callback[0])
    {    
        LPTMR_Callback[0]();     
    }
}

