/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file     lptmr.h
*
* @author   Freescale
*
* @version  0.0.1
*
* @date     Jun 25, 2013
*
* @brief    Low-Power Timer(LPTMR) driver head file. 
*
******************************************************************************/
#ifndef _LPTMR_H_
#define _LPTMR_H_
#ifdef __cplusplus
extern "C" {
#endif
/******************************************************************************
* Includes
******************************************************************************/

/******************************************************************************
* Constants
******************************************************************************/

/******************************************************************************
* Macros
******************************************************************************/
/******************************************************************************
* LPTMR control bit definition
*
*//*! @addtogroup lptmr_controlbit
* @{
*******************************************************************************/

#define LPTMR_SRC_MCGIRCLK      0 	/*!<  LPTMR clock 0*/ 
#define LPTMR_SRC_LPO           1 	/*!<  LPTMR clock 1*/ 
#define LPTMR_SRC_ERCLK32K      2 	/*!<  LPTMR clock 2*/ 
#define LPTMR_SRC_MOSCERCLK     3 	/*!<  LPTMR clock 3*/ 
    
#define LPTMR_PULSE_INPUT_CMP0OUT  0 	/*!<  pulse input 0*/ 
#define LPTMR_PULSE_INPUT_ALT1     1 	/*!<  pulse input 1*/     
#define LPTMR_PULSE_INPUT_ALT2     2 	/*!<  pulse input 2*/     
#define LPTMR_PULSE_INPUT_ALT3     3 	/*!<  pulse input 3*/ 

#define PRESCALE_DIVIDER_2                 0    /*!<  divider is 2*/ 
#define PRESCALE_DIVIDER_4              0x01    /*!<  divider is 4*/     
#define PRESCALE_DIVIDER_8              0x02    /*!<  divider is 8*/      
#define PRESCALE_DIVIDER_16             0x03    /*!<  divider is 16*/      
#define PRESCALE_DIVIDER_32             0x04    /*!<  divider is 32*/      
#define PRESCALE_DIVIDER_64             0x05    /*!<  divider is 64*/      
#define PRESCALE_DIVIDER_128            0x06    
#define PRESCALE_DIVIDER_256            0x07    
#define PRESCALE_DIVIDER_512            0x08     
#define PRESCALE_DIVIDER_1024           0x09      
#define PRESCALE_DIVIDER_2048           0x0A    
#define PRESCALE_DIVIDER_4096           0x0B    
#define PRESCALE_DIVIDER_8192           0x0C    
#define PRESCALE_DIVIDER_16384          0x0D      
#define PRESCALE_DIVIDER_32768          0x0E          
#define PRESCALE_DIVIDER_65536          0x0F        
    
/*! @} End of lptmr_controlbit                                                  */

/******************************************************************************
* Types
******************************************************************************/

/*
 * Callback type
 */

/******************************************************************************
* LPTMR callback function declaration
*
*//*! @addtogroup lptmr_callback
* @{
*******************************************************************************/
typedef void (*LPTMR_CallbackType)(void);             /*!< LPTMR callback type */

/*! @} End of lptmr_callback                                                    */



/******************************************************************************
* LPTMR configuration structre.
*
*//*! @addtogroup LPTMR_ConfigType
* @{
*******************************************************************************/

/* LPTMR configuration structure 
 */  
/*!
 * @brief LPTMR configuration type.
 *
 */
typedef struct
{   
    /* LPTMRx_CSR register setting bit */
    uint32_t bLPTMREn            : 1 ;    /*!< 1: LPTMR is enabled, 0: LPTMR is disabled */ 
    uint32_t bTimerMode          : 1 ;    /*!< 1: puse counter mode, 0: timer counter mode */
    uint32_t bFreeRuning         : 1 ;    /*!< 1: counter is reset when counter overflow , 0: counter is reset whenever TCF is set */
    uint32_t bPinPolarity        : 1 ;    /*!< 1: CNR will increase on falling-edge , 0: CNR will increase on rising-edge */
    uint32_t bPinSelect          : 2 ;    /*!< select pulse ounter input 0~3*/
    uint32_t bInterruptEn        : 1 ;    /*!< 1: LPTMR interrupt is enable, 0: LPTMR interrupt is disable */     
    uint32_t bFlag               : 1 ;    /*!< 1: 1: flag is set,and write 0 to clear flag, 0: no flag is set */ 
    /* LPTMRx_PSR register setting bit */ 
    uint32_t bClockSelect        : 2 ;    /*!< select colck source of prescaler */ 
    uint32_t bPrescaleBypassEn   : 1 ;    /*!< 1: presacler is bypassed, 0: LPTMR is enabled */     
    uint32_t bPrescaleSelect     : 4 ;    /*!< prescaler value from 2 to 65536 */
    /* LPTMRx_PCMR register setting bit */ 
    uint16_t  u16CompareValue    ;        /*!< timer compare value*/ 
} LPTMR_ConfigType, *LPTMR_ConfigPtr; 
/*! @} LPTMR_ConfigType */

/******************************************************************************
* Global variables
******************************************************************************/


/*!
 * inline functions
 */

/******************************************************************************
* LPTMR API list
*
*//*! @addtogroup lptmr_api_list
* @{
*******************************************************************************/

/*****************************************************************************//*!
*
* @brief enable lptmr module.
*        
* @param   pLPTMR point to lptmr module type
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void LPTMR_Enable(LPTMR_Type *pLPTMR)
{

    pLPTMR->CSR |= LPTMR_CSR_TEN_MASK; 
    
}


/*****************************************************************************//*!
*
* @brief disable lptmr module.
*        
* @param  pLPTMR point to lptmr module type
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void LPTMR_Disable(LPTMR_Type *pLPTMR)
{

    pLPTMR->CSR &= ~LPTMR_CSR_TEN_MASK; 
}

/*****************************************************************************//*!
*
* @brief enable lptmr interrupt.
*        
* @param  pLPTMR point to lptmr module type
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void LPTMR_IntEnable(LPTMR_Type *pLPTMR)
{

    pLPTMR->CSR |= LPTMR_CSR_TIE_MASK; 
}

/*****************************************************************************//*!
*
* @brief disable lptmr interrupt.
*        
* @param  pLPTMR point to lptmr module type
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void LPTMR_IntDisable(LPTMR_Type *pLPTMR)
{

    pLPTMR->CSR &= ~LPTMR_CSR_TIE_MASK; 
}

/*****************************************************************************//*!
*
* @brief get lptmr compare flag.
*        
* @param[in] pLPTMR  point to lptmr module type
*
* @return bflag.
*
* @ Pass/ Fail criteria: none

*****************************************************************************/
__STATIC_INLINE uint8_t LPTMR_GetFlags(LPTMR_Type *pLPTMR)

{
    uint8_t bflag;  
    
    bflag = (pLPTMR->CSR & LPTMR_CSR_TCF_MASK);    
    
    return bflag; 
}

/*****************************************************************************//*!
*
* @brief clear lptmr compare flag.
*        
* @param[in] pLPTMR  point to lptmr module type
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void LPTMR_ClrFlags(LPTMR_Type *pLPTMR)
{  
    pLPTMR->CSR |= LPTMR_CSR_TCF_MASK;       
}

/*****************************************************************************//*!
*
* @brief set pulse counter mode.
*        
* @param[in] pLPTMR  point to lptmr module type
*
* @return none.
*
* @ Pass/ Fail criteria: none

*****************************************************************************/
__STATIC_INLINE void LPTMR_SetPulseMode(LPTMR_Type *pLPTMR)

{
    pLPTMR->CSR |= LPTMR_CSR_TMS_MASK;      
}

/*****************************************************************************//*!
*
* @brief set time counter mode.
*        
* @param[in] pLPTMR  point to lptmr module type
*
* @return none.
*
* @ Pass/ Fail criteria: none

*****************************************************************************/
__STATIC_INLINE void LPTMR_SetTimeMode(LPTMR_Type *pLPTMR)

{
    pLPTMR->CSR &= ~LPTMR_CSR_TMS_MASK;      
}

/*****************************************************************************//*!
*
* @brief set free-running counter.
*        
* @param[in] pLPTMR  point to lptmr module type
*
* @return none.
*
* @ Pass/ Fail criteria: none

*****************************************************************************/
__STATIC_INLINE void LPTMR_SetFreeRunning(LPTMR_Type *pLPTMR)

{
    pLPTMR->CSR |= LPTMR_CSR_TFC_MASK;      
}

/*****************************************************************************//*!
*
* @brief set pin active high polarity.
*        
* @param[in] pLPTMR  point to lptmr module type
*
* @return none.
*
* @ Pass/ Fail criteria: none

*****************************************************************************/
__STATIC_INLINE void LPTMR_SetPinHigh(LPTMR_Type *pLPTMR)

{
    pLPTMR->CSR &= ~LPTMR_CSR_TPP_MASK;      
}

/*****************************************************************************//*!
*
* @brief set pin active low polarity.
*        
* @param[in] pLPTMR  point to lptmr module type
*
* @return none.
*
* @ Pass/ Fail criteria: none

*****************************************************************************/
__STATIC_INLINE void LPTMR_SetPinLow(LPTMR_Type *pLPTMR)

{
    pLPTMR->CSR |= LPTMR_CSR_TPP_MASK;      
}

/*****************************************************************************//*!
*
* @brief set pin select
*        
* @param[in] pLPTMR  point to lptmr module type
*
* @return none.
*
* @ Pass/ Fail criteria: none

*****************************************************************************/
__STATIC_INLINE void LPTMR_PinSelect(LPTMR_Type *pLPTMR, uint8_t u8PinsNumber)

{
    pLPTMR->CSR &= ~LPTMR_CSR_TPS_MASK;    
    pLPTMR->CSR |=  LPTMR_CSR_TPS(u8PinsNumber);
}

/*****************************************************************************//*!
*
* @brief set prescale value
*        
* @param[in] pLPTMR  point to lptmr module type
*
* @return none.
*
* @ Pass/ Fail criteria: none

*****************************************************************************/
__STATIC_INLINE void LPTMR_PrescaleSelect(LPTMR_Type *pLPTMR, uint8_t u8PrescaleValue)
{
    pLPTMR->PSR &= ~LPTMR_PSR_PRESCALE_MASK;    
    pLPTMR->PSR |=  LPTMR_PSR_PRESCALE(u8PrescaleValue);
}

/*****************************************************************************//*!
*
* @brief bypass prescale
*        
* @param[in] pLPTMR  point to lptmr module type
*
* @return none.
*
* @ Pass/ Fail criteria: none

*****************************************************************************/
__STATIC_INLINE void LPTMR_PrescaleBypass(LPTMR_Type *pLPTMR)
{
    pLPTMR->PSR |= LPTMR_PSR_PBYP_MASK;    
}

/*****************************************************************************//*!
*
* @brief enable prescale
*        
* @param[in] pLPTMR  point to lptmr module type
*
* @return none.
*
* @ Pass/ Fail criteria: none

*****************************************************************************/
__STATIC_INLINE void LPTMR_PresacleEnalbe(LPTMR_Type *pLPTMR)
{
    pLPTMR->PSR &= ~LPTMR_PSR_PBYP_MASK;    
}

/*****************************************************************************//*!
*
* @brief select presacler clock
*        
* @param[in] pLPTMR  point to lptmr module type
*
* @return none.
*
* @ Pass/ Fail criteria: none

*****************************************************************************/
__STATIC_INLINE void LPTMR_ClockSelect(LPTMR_Type *pLPTMR, uint8_t u8ClockSelect)
{
    pLPTMR->PSR &= ~LPTMR_PSR_PCS_MASK;    
    pLPTMR->PSR |=  LPTMR_PSR_PCS(u8ClockSelect);
}

/*****************************************************************************//*!
*
* @brief get lptmr counter.
*        
* @param[in] pLPTMR  point to lptmr module type
*
* @return bflag.
*
* @ Pass/ Fail criteria: none

*****************************************************************************/
__STATIC_INLINE uint16_t LPTMR_GetCounter(LPTMR_Type *pLPTMR)

{
    uint16_t u16CounterValue=0; 
 
    pLPTMR->CNR = u16CounterValue; /*!< it is necessary to write any value to CNR before read the CNR*/ 
    u16CounterValue = pLPTMR->CNR; 
    
    return u16CounterValue; 
}


/******************************************************************************
* Global functions
******************************************************************************/
void LPTMR_Init(LPTMR_Type *pLPTMR, LPTMR_ConfigType *pConfig);
void LPTMR_DeInit(void);
/*! @} End of lptmr_api_list                                                   */

#ifdef __cplusplus
}
#endif
#endif /* _LPTMR_H_ */
