
/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file mmdvsq.c
*
* @author Freescale
*
* @version 0.0.1
*
* @date Aug. 25, 2013
*
* @brief providing APIs for h/w divide and sqrt (MMDVSQ)
*
*******************************************************************************
*
* provide APIs for h/w divide and sqrt (MMDVSQ)
******************************************************************************/
#include "common.h"
#include "mmdvsq.h"

/******************************************************************************
* Global variables
******************************************************************************/

/******************************************************************************
* Constants and macros
******************************************************************************/

/******************************************************************************
* Local types
******************************************************************************/

/******************************************************************************
* Local function prototypes
******************************************************************************/


/******************************************************************************
* Local variables
******************************************************************************/

/******************************************************************************
* Local functions
******************************************************************************/

/******************************************************************************
* Global functions
******************************************************************************/


/*****************************************************************************//*!
*
* @brief configure divide operation mode.
*        
* @param[in]  pDivideConfig  pointer to divide configuration structure
*
* @return None
*
* @ Pass/ Fail criteria: none
* @see  MMDVSQ_DivideConfigType
*****************************************************************************/

void    MMDVSQ_ConfigDivide(MMDVSQ_DivideConfigPtr pDivideConfig)
{
   uint32_t    csr = 0;  /*  control bits */
    
    if(pDivideConfig->bUsignedDivide)
    {
        csr |= MMDVSQ_CSR_USGN_MASK;    
    }
    if(pDivideConfig->bCalculateRemainder)
    {
        csr |= MMDVSQ_CSR_REM_MASK;    
    }
    if(pDivideConfig->bDisableFastStart)
    {
        csr |= MMDVSQ_CSR_DFS_MASK;    
    }

    if(pDivideConfig->bEnableDivideBy0)
    {
        csr |= MMDVSQ_CSR_DZE_MASK;            
    }
  
    MMDVSQ->CSR = csr;
}





