/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file     pdb.c
*
* @author   Freescale
*
* @version  0.0.1
*
* @date     Jun 25, 2013
*
* @brief    Programmable Delay Block (PDB) source code. 
*
******************************************************************************/
#include "common.h"
#include "pdb.h"

/******************************************************************************
* Global variables
******************************************************************************/

/******************************************************************************
* Constants and macros
******************************************************************************/

/******************************************************************************
* Local types
******************************************************************************/

/******************************************************************************
* Local function prototypes
******************************************************************************/

/******************************************************************************
* Local variables
******************************************************************************/
/*!
 * @brief global variable to store PDB callbacks.
 *
 */
PDB_CallbackType PDB_Callback[2] = {(PDB_CallbackType)NULL}; /*!< PDB initial callback */

/******************************************************************************
* Local functions
******************************************************************************/
void PDB_Ch0Isr(void);
void PDB_Ch1Isr(void);


/******************************************************************************
* Global functions
******************************************************************************/

/******************************************************************************
* define PDB APIs
*
*//*! @addtogroup pdb_api_list
* @{
*******************************************************************************/

/*****************************************************************************//*!
*
* @brief initialize pdb module.
*        
* @param[in]   u8Channel_No channel number
* @param[in]   pConfig point to configuration  
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void PDB_Init(PDB_ConfigType *pConfig)
{  
    SIM->SCGC6         |= SIM_SCGC6_PDB_MASK;         /*!< enable PDB clock gate */

    PDB0->SC |= PDB_SC_MULT(pConfig->bMultiFactor);  /*!< mutliplication factor for presacler */   
    PDB0->SC |= PDB_SC_TRGSEL(pConfig->bTriggerSource);  /*!< trigger source select */ 
    PDB0->SC |= PDB_SC_PRESCALER(pConfig->bPrescaler);  /*!< presclaer select */ 

    PDB0->CH[0].C1 |= PDB_C1_EN(pConfig->u8Ch0PretriggerEn) | PDB_C1_TOS(pConfig->u8Ch0PretriggerOut) | PDB_C1_BB(pConfig->u8Ch0Back2BackEn);
    
    PDB0->CH[1].C1 |= PDB_C1_EN(pConfig->u8Ch1PretriggerEn) | PDB_C1_TOS(pConfig->u8Ch1PretriggerOut) | PDB_C1_BB(pConfig->u8Ch1Back2BackEn);    

    PDB0->DAC[0].INT = pConfig->u16DACInterval;    

    if (pConfig->bPDBEn)            
    {                                               
        PDB_Enable();                       /*!< enable pdb module */           
    }     
    /*load register value*/
    //PDB0->SC           |= PDB_SC_LDMOD(pConfig->bLoadMode);
    PDB0->MOD          = pConfig->u16ModulusValue;
    PDB0->IDLY         = pConfig->u16InterruptDelay ;
    
    PDB0->CH[0].DLY[0] = pConfig->u16Ch0Delay0;
    PDB0->CH[0].DLY[1] = pConfig->u16Ch0Delay1;
    PDB0->CH[1].DLY[0] = pConfig->u16Ch1Delay0;
    PDB0->CH[1].DLY[1] = pConfig->u16Ch1Delay1;     
    PDB0->SC          |= PDB_SC_LDOK_MASK | PDB_SC_LDMOD(pConfig->bLoadMode);  /*!< load value */ 
       
    if (pConfig->bContinuousMode) 
    {                                              
     
        PDB0->SC |= PDB_SC_CONT_MASK;       /*!< enable PDB continuous operation mode */
    } 
    
    if (pConfig->bDMAEn )            
    {                                            
        PDB0->SC |= PDB_SC_DMAEN_MASK;          /*!< DMA request is enable */                   
    } 
           
    if (pConfig->bErrorInterruptEn )            
    {                                                 
        PDB0->SC |= PDB_SC_PDBEIE_MASK;          /*!< enable PDB sequence error interrupt */                    
    }
      
    if (pConfig->bInterruptEn )            
    {                                                 
        enable_irq(PDB0_IRQn);          /*!< enable PDB vector */                   
    }                   
    else                                     
    {                     
        NVIC_DisableIRQ(PDB0_IRQn);         /*!< disable PDB vector */        
    }
    
    if (pConfig->bDACTriggerEn )            
    {                                                 
      
        PDB0->DAC[0].INTC |= PDB_INTC_TOE_MASK;
        /*!< enable PDB vector */                   
    }
    


}

/*****************************************************************************//*!
*
* @brief pdb module set call back.
*        
* @param[in] u8Channel_No channel number.   
* @param[in] pfnCallback point to call back.   
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void PDB_SetCallback(uint8_t u8Channel_No, PDB_CallbackType pfnCallback)
{
  PDB_Callback[u8Channel_No] = pfnCallback;
}


/*! @} End of pdb_api_list                                                   */


/*****************************************************************************//*!
*
* @brief pdb module channel 0 isr.
*        
* @param none
*
* @return none 
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void PDB_Ch0Isr(void) 
{   
   //PDB_ChannelClrFlags(0);  
    
    if (PDB_Callback[0])
    {    
        PDB_Callback[0]();     
    }
}
    
/*****************************************************************************//*!
*
* @brief pdb module channel 1 isr.
*        
* @param none
*
* @return none 
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void PDB_Ch1Isr(void) 
{   
    //PDB_ChannelClrFlags(1);
    if (PDB_Callback[1])
    {    
        PDB_Callback[1]();     
    }
}    


