/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file     pdb.h
*
* @author   Freescale
*
* @version  0.0.1
*
* @date     Jun 25, 2013
*
* @brief    Programmable Delay Block (PDB) driver head file. 
*
******************************************************************************/
#ifndef _PDB_H_
#define _PDB_H_
#ifdef __cplusplus
extern "C" {
#endif
/******************************************************************************
* Includes
******************************************************************************/

/******************************************************************************
* Constants
******************************************************************************/

/******************************************************************************
* PDB channel number list
*
*//*! @addtogroup pdb_channelnumber
* @{
*******************************************************************************/
enum
{
    PDB_CHANNEL0 = 0,       /*!< PDB channel 0 */
    PDB_CHANNEL1            /*!< PDB channel 1 */
};

/*! @} End of pdb_channelnumber                                               */

/******************************************************************************
* Macros
******************************************************************************/
/******************************************************************************
* PDB control bit definition
*
*//*! @addtogroup pdb_controlbit
* @{
*******************************************************************************/

#define PDB_TRIGGER_LPTMR          0x0E	/*!<  LPTMR trigger source number*/ 



/*! @} End of pdb_controlbit                                                  */

/******************************************************************************
* Types
******************************************************************************/

/*
 * Callback type
 */

/******************************************************************************
* PDB callback function declaration
*
*//*! @addtogroup pdb_callback
* @{
*******************************************************************************/
typedef void (*PDB_CallbackType)(void);             /*!< PDB callback type */

/*! @} End of pdb_callback                                                    */



/******************************************************************************
* PDB configuration structre.
*
*//*! @addtogroup PDB_ConfigType
* @{
*******************************************************************************/

/* PDB configuration structure 
 */  
/*!
 * @brief PDB configuration type.
 *
 */
typedef struct
{   
    /* PDBx_SC register setting bit */
    uint32_t bLoadOk             : 1 ;    /*!< 1: write 1 to update internal register */
    uint32_t bContinuousMode     : 1 ;    /*!< 1: PDB module is in continue mode, 0: PDB module is in one-shot mode */    
    uint32_t bMultiFactor        : 2 ;    /*!< multiplication factor select for prescaler */ 
    uint32_t bReserved0          : 1 ;    /*!< reserved bit */ 
    uint32_t bInterruptEn        : 1 ;    /*!< 1: PDB interrupt is enable, 0: PDB interrupt is disable */     
    uint32_t bFlag               : 1 ;    /*!< 1: 1: flag is set,and write 0 to clear flag, 0: no flag is set */ 
    uint32_t bPDBEn              : 1 ;    /*!< 1: PDB is enable, 0: PDB is disable */ 
    uint32_t bTriggerSource      : 4 ;    /*!< trigger souce select */ 
    uint32_t bPrescaler          : 3 ;    /*!< Prescaler Divider Select */ 
    uint32_t bDMAEn              : 1 ;    /*!< 1: DMA request is enable, 0: DMA request is disable */
    uint32_t bSoftwareTrigger    : 1 ;    /*!< 1: reset and restart counter, 0: write 0 is no effect */ 
    uint32_t bErrorInterruptEn   : 1 ;    /*!< 1: PDB sequence error interrupt enabled, 0: PDB sequence error interrupt disabled*/         
    uint32_t bLoadMode           : 2 ;    /*!< load mode select */ 
    uint32_t bReserved1          : 12;    /*!< reserved bit */ 
    
    /* PDBx_MOD register setting bit */    
    uint16_t u16ModulusValue         ;    /*!< 16-bit PDB modulus value */

    /* PDBx_IDLY register setting bit */       
    uint16_t u16InterruptDelay       ;    /*!< 16-bit PDB interrupt delay time */ 
    
    /* PDBx_CHnC1 register setting bit */     
	uint8_t  u8Ch0PretriggerEn    ;       /*!< channel0 pre-trigger enable */
	uint8_t  u8Ch1PretriggerEn    ;       /*!< channel1 pre-trigger enable */    
    uint8_t  u8Ch0PretriggerOut   ;       /*!< channel0 pre-trigger output select */
    uint8_t  u8Ch1PretriggerOut   ;       /*!< channel1 pre-trigger output select */    
    uint8_t  u8Ch0Back2BackEn     ;       /*!< channel 0 back-to-back operation enable */ 
    uint8_t  u8Ch1Back2BackEn     ;       /*!< channel 1 back-to-back operation enable */ 
    
    /* PDBx_CHnS register setting bit */       
    uint8_t  u8ErrorFlag          ;       /*!< channel flag, write 0 to clear  */ 
    uint8_t  u8ChannelFlag        ;       /*!< channel error flag, write 0 to clear */ 
    
    /* PDBx_IDLY register setting bit */      
    uint16_t u16Ch0Delay0       ;       /*!< channel0 delay time */
    uint16_t u16Ch0Delay1       ;       /*!< channel0 delay time */    
    uint16_t u16Ch1Delay0       ;       /*!< channel1 delay time */ 
    uint16_t u16Ch1Delay1       ;       /*!< channel1 delay time */
    
    uint32_t bDACTriggerEn     :1;       /*!< 1: DAC interval trigger is enable, 0: DAC interval trigger is disable*/
    uint32_t bDACTriggerInput  :1;       /*!< 1: DAC interval trigger input is enable, 0: DAC interval trigger is disable */
    uint16_t u16DACInterval        ;     /*!< DAC interval value*/        
    uint8_t  u8PulseOutEn          ;     /*!< pulse-out enable */  
    uint16_t  u16PulseOutDelay1    ;     /*!< pulse-out delay1*/ 
    uint16_t  u16PulseOutDelay2    ;     /*!< pulse-out delay2*/    
} PDB_ConfigType, *PDB_ConfigPtr; 
/*! @} PDB_ConfigType */

/******************************************************************************
* Global variables
******************************************************************************/


/*!
 * inline functions
 */

/******************************************************************************
* PDB API list
*
*//*! @addtogroup pdb_api_list
* @{
*******************************************************************************/

/*****************************************************************************//*!
*
* @brief enable pdb module.
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void PDB_Enable(void)
{

    PDB0->SC |= PDB_SC_PDBEN_MASK; 
    
}


/*****************************************************************************//*!
*
* @brief disable pdb module.
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void PDB_Disable(void)
{

    PDB0->SC &= ~PDB_SC_PDBEN_MASK;  
}

/*****************************************************************************//*!
*
* @brief enable pdb channel timer.
*        
* @param[in]   u8Channel.
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void PDB_ChannelEnable(uint8_t u8Channel)

{

    PDB0->CH[u8Channel].C1 |= PDB_C1_EN_MASK; 
}


/*****************************************************************************//*!
*
* @brief disable pdb channel timer.
*        
* @param[in]   u8Channel.
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
__STATIC_INLINE void PDB_ChannelDisable(uint8_t u8Channel)
{

    PDB0->CH[u8Channel].C1 &= ~PDB_C1_EN_MASK;  
}

/******************************************************************************
* Global functions
******************************************************************************/
void PDB_Init(PDB_ConfigType *pConfig);
void PDB_DeInit(void);
/*! @} End of pdb_api_list                                                   */

#ifdef __cplusplus
}
#endif
#endif /* _PDB_H_ */
