
/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file wdog.c
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun. 25, 2013
*
* @brief Provide common watchdog module routines. 
*
* @history:
* 	Jun. 25, 2013	modified the watch dog unlock sequence and disable sequence
******************************************************************************/
#include "common.h"
#include "wdog.h"
 
/******************************************************************************
* Global variables
******************************************************************************/

/******************************************************************************
* Constants and macros
******************************************************************************/

/******************************************************************************
* Local types
******************************************************************************/

/******************************************************************************
* Local function prototypes
******************************************************************************/

/******************************************************************************
* Local variables
******************************************************************************/

/******************************************************************************
* Local functions
******************************************************************************/

/******************************************************************************
* Global functions
******************************************************************************/

/*****************************************************************************//*!
*
* @brief Watchdog timer disable routine.
*        
* @param  none 
*
* @return none
*
* @ Pass/ Fail criteria: none
* @see WDOG_Enable
*****************************************************************************/

void WDOG_Disable(void)
{
	/* First unlock the watchdog so that we can write to registers */
    WDOG_Unlock();
    
	/* Clear the WDOGEN bit to disable the watchdog */
	WDOG->STCTRLH &= ~WDOG_STCTRLH_WDOGEN_MASK;
}


/*****************************************************************************//*!
*
* @brief Watchdog timer disable routine with update enabled.
*        
*   Disable watchdog but the watchdog can be enabled and updated later.
*
* @param  none 
*
* @return none
*
* @ Pass/ Fail criteria: none
* @see WDOG_Enable
*****************************************************************************/

void WDOG_DisableUpdateEnable(void)
{
	/* First unlock the watchdog so that we can write to registers */
    //WDOG_Unlock();
    
    //WDOG->CS2    |= WDOG_CS2_DEFAULT_VALUE;
    //WDOG_TOVAL  =  WDOG_TOVAL_DEFAULT_VALUE;
    //WDOG_WIN    =  WDOG_WIN_DEFAULT_VALUE;    
    //WDOG->CS1   =  WDOG_CS1_UPDATE_MASK;
}

/*****************************************************************************//*!
*
* @brief Watchdog timer enable routine with default values.
*        
* @param[in]  u16TimeOut    timeout value.
*
* @return none
*
* @ Pass/ Fail criteria: none
* @see  WDOG_Disable
*****************************************************************************/

void WDOG_Enable(uint16 u16TimeOut)
{
	/* First unlock the watchdog so that we can write to registers */
    //WDOG_Unlock();
    
    //WDOG->CS2    |= WDOG_CS2_DEFAULT_VALUE;
    //WDOG->TOVALL  =  u16TimeOut;
   // WDOG->TOVALH  =  u16TimeOut >> 8;
   // WDOG_WIN    =  WDOG_WIN_DEFAULT_VALUE;    
    //WDOG->CS1   = WDOG_CS1_EN_MASK;
}


/*****************************************************************************//*!
*
* @brief initialize watchdog.
*        
* @param[in]   pConfig  poiner to watchdog configuration strcture.
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/

void WDOG_Init(WDOG_ConfigPtr pConfig)
{
 
#if 0
    uint8_t     u8Cs1;
    uint8_t     u8Cs2;
    uint16_t    u16Toval;
    uint16_t    u16Win;
    
    u8Cs1       = 0x80;                                   /* default CS1 register value */
    u8Cs2       = 0;
    u16Toval    = pConfig->u16TimeOut;
    u16Win      = pConfig->u16WinTime;
    
    if(pConfig->sBits.bDisable)
    {
        u8Cs1 &= ~WDOG_CS1_EN_MASK;
    }
    if(pConfig->sBits.bIntEnable)
    {
        u8Cs1 |= WDOG_CS1_INT_MASK;
    }
    if(pConfig->sBits.bStopEnable)
    {
        u8Cs1 |= WDOG_CS1_STOP_MASK;
    }
    if(pConfig->sBits.bDbgEnable)
    {
        u8Cs1 |= WDOG_CS1_DBG_MASK;
    }
    if(pConfig->sBits.bWaitEnable)
    {
        u8Cs1 |= WDOG_CS1_WAIT_MASK;
    }
    if(pConfig->sBits.bUpdateEnable)
    {
        u8Cs1 |= WDOG_CS1_UPDATE_MASK;
    }
    if(pConfig->sBits.bWinEnable)
    {
        u8Cs2 |= WDOG_CS2_WIN_MASK;
    }
    if(pConfig->sBits.bPrescaler)
    {
        u8Cs2 |= WDOG_CS2_PRES_MASK;
    }
    u8Cs2   |= (pConfig->sBits.bClkSrc & 0x03);
    
    /* write regisers */
    WDOG_Unlock();              /* unlock watchdog first */
    WDOG->CS2   = u8Cs2;
    WDOG->TOVALL = u16Toval;
    WDOG->TOVALH = u16Toval >> 8;
    WDOG->WINL    = u16Win;
    WDOG->WINH    = u16Win >> 8;
    WDOG->CS1   = u8Cs1;                            
#endif
}


/*****************************************************************************//*!
*
* @brief feed/refresh watchdog.
*        
* @param   none
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/

void WDOG_Feed(void)
{
   // DisableInterrupts;
   // WDOG_CNT = 0x02A6;
   // WDOG_CNT = 0x80B4;
   // EnableInterrupts;
}



/*****************************************************************************//*!
*
* @brief enable update of WDOG. 
*        
* @param  none
*
* @return none
*
* @ Pass/ Fail criteria: none
* @warning  this must be the last step of writing control bits sequence.
*****************************************************************************/

void WDOG_EnableUpdate(void)
{
    //WDOG_Unlock(); 
    //WDOG->CS1 |= WDOG_CS1_UPDATE_MASK;
}


/*****************************************************************************//*!
*
* @brief disable update of WDOG. 
*        
* @param  none 
*
* @return none
*
* @ Pass/ Fail criteria: none
* @warning  this must be the last step of writing control bits sequence.
*****************************************************************************/

void WDOG_DisableUpdate(void)
{
    //WDOG_Unlock(); 
    //WDOG->CS1 &= ~WDOG_CS1_UPDATE_MASK;
}


/********************************************************************/




