/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2004-2011 Freescale Semiconductor
* ALL RIGHTS RESERVED.
*
****************************************************************************//*!
*
* @file   peripherals_init.c
*
* @brief  Init functions
*        
* @version 1.0.3.0
* 
* @date Nov-27-2013
* 
*******************************************************************************/
#include "peripherals_init.h"
#include "hwconfig.h"

/******************************************************************************
* Constants and macros
******************************************************************************/
#define TWO_POWER_N_SAMPLES  6  /* select carefully - COP timer not updated */

/******************************************************************************
* Local variables
******************************************************************************/
const Pwm_sChannelControl bldcCommutationTableComp[8] = { // mask, swap
                                                    {0x30,0x02},    // [0] - sector 0
                                                    {0x0C,0x04},    // [1] - sector 1
                                                    {0x03,0x04},    // [2] - sector 2
                                                    {0x30,0x01},    // [3] - sector 3
                                                    {0x0C,0x01},    // [4] - sector 4
                                                    {0x03,0x02},    // [5] - sector 5
                                                    {0x00,0x06},    // [6] - alignment vector (combination of sectors 0 & 1)
                                                    {0x3F,0x00},    // [7] - PWM Off
                                                  };

const unsigned short bldcAdcSectorCfg[8] = {ADC_CHANNEL_PHASEC, ADC_CHANNEL_PHASEB, ADC_CHANNEL_PHASEA,
                                            ADC_CHANNEL_PHASEC, ADC_CHANNEL_PHASEB, ADC_CHANNEL_PHASEA,
                                            ADC_CHANNEL_PHASEA, ADC_CHANNEL_PHASEA
                                           };   
/*
LED - PTD6 - ALT1

Brake control - PTE24  - ALT1
USER LED + SW2 - PTB0  - ALT1 (input)
SW1           - PTA4   - ALT1 (input)

MC33937 RESET   - PTC2  - ALT1
MC33937 DRV_EN  - PTD7  - ALT1
MC33937 DRV_INT - PTB3  - ALT1 (input)
MC33937 DRV_OC  - PTB2  - ALT1 (input)

SPI0_MISO - PTD3 - ALT2
SPI0_MOSI - PTD2 - ALT2
SPI0_/SS  - PTC0 - ALT7
SPI0_SCLK - PTC5 - ALT2

PWM0 - AT - PTC1  - ALT4
PWM1 - AB - PTE25 - ALT3
PWM2 - BT - PTC3  - ALT4
PWM3 - BB - PTC4  - ALT4
PWM4 - CT - PTD4  - ALT4
PWM5 - CB - PTD5  - ALT4
*/

/*************************************************************************
 * Function Name: Clock_init
 * Parameters: none
 * Return: none
 * Description: Clock initialization
 *************************************************************************/
void Clock_init(void)
{
  // Enable clock to all peripherals
  SIM->SCGC4 = SIM_SCGC4_EWM_MASK | SIM_SCGC4_I2C0_MASK | SIM_SCGC4_UART0_MASK | SIM_SCGC4_UART1_MASK | SIM_SCGC4_CMP_MASK;
  SIM->SCGC5 = SIM_SCGC5_LPTMR_MASK | SIM_SCGC5_PORTA_MASK | SIM_SCGC5_PORTB_MASK | SIM_SCGC5_PORTC_MASK | SIM_SCGC5_PORTD_MASK | SIM_SCGC5_PORTE_MASK;
  SIM->SCGC6 = SIM_SCGC6_FTF_MASK | SIM_SCGC6_DMAMUX_MASK | SIM_SCGC6_SPI0_MASK | SIM_SCGC6_CRC_MASK | SIM_SCGC6_PDB_MASK | \
              SIM_SCGC6_FTM0_MASK | SIM_SCGC6_FTM1_MASK | SIM_SCGC6_FTM2_MASK | SIM_SCGC6_ADC0_MASK | SIM_SCGC6_ADC1_MASK | SIM_SCGC6_DAC0_MASK; 
  SIM->SCGC7 = SIM_SCGC7_DMA_MASK;

  // clock setting to 75 MHz
  //SIM_CLKDIV1 = SIM_CLKDIV1_OUTDIV1(0) | SIM_CLKDIV1_OUTDIV4(2) ;  // system clock = MCGOUTCLK ; Flash Clock (bus clock) = MCGOUTCLK / 3
  //MCG_C4 |= MCG_C4_DRST_DRS(2); //Mid-high Frequency Range - 6075 MHz
  //fei_fee(10000000, 0, 1);// transition from FEI to FEE mode
  //clk_monitor_0(1); // clock monitor enable
  
  // clock setting to 72 MHz, internal 32768 Hz reference clock oscilator
  SIM->CLKDIV1 = SIM_CLKDIV1_OUTDIV1(0) | SIM_CLKDIV1_OUTDIV4(2) ;  // system clock = MCGOUTCLK ; Flash Clock (bus clock) = MCGOUTCLK / 3
  MCG->C4 |= MCG_C4_DRST_DRS(2) | MCG_C4_DMX32_MASK; //Mid-high Frequency Range - 6075 MHz, reference 32768 Hz, PLL factor 2197, 72 MHz
}

/*************************************************************************
 * Function Name: ADC0_init
 * Parameters: none
 * Return: none
 * Description: ADC0 module initialization
 *************************************************************************/
int ADC0_init(void)
{
long tmp=0;
unsigned long tmp_1[7] = {0,0,0,0,0,0,0}, tmp_2[7] = {0,0,0,0,0,0,0}; 
register uint16 i = 0, numLoops = ((uint16) 1 << TWO_POWER_N_SAMPLES);
tADC_CALIB adc0_calib;

  /* ADC calibration */

  // Bus clock / 1 (24 MHz), 12-bit, Bus clock source
  ADC0->CFG1 = ADC_CFG1_ADIV(0) | ADC_CFG1_MODE(1) | ADC_CFG1_ADICLK(0);
  // high-speed mode, 2 extra ADC clock cycles; 6 ADCK cycles sample time
  ADC0->CFG2 = ADC_CFG2_ADHSC_MASK | ADC_CFG2_ADLSTS(3);
  ADC0->SC2  = ADC_SC2_REFSEL(0);
  
  /* calibration loop                                                       */
  while ((i++) < numLoops)
  { 
    /* The input channel, conversion mode continuous function, compare      */
    /* function, resolution mode, and differential/single-ended mode are    */
    /* all ignored during the calibration function.                         */
    ADC0->SC1[0] = 0x1f;
    ADC0->SC3  = ADC_SC3_AVGS(0) & (~ADC_SC3_CAL_MASK);
    ADC0->SC3 |= ADC_SC3_CAL_MASK;
    while (!(ADC0->SC1[0] & ADC_SC1_COCO_MASK));
    
    if (ADC0->SC3 & ADC_SC3_CALF_MASK) { ADC0->SC3 |= ADC_SC3_CALF_MASK; return 1; }
    
    tmp+= (short int) ADC0->OFS;
    
    tmp_1[0]+=(unsigned long)ADC0->CLP0;
    tmp_1[1]+=(unsigned long)ADC0->CLP1;
    tmp_1[2]+=(unsigned long)ADC0->CLP2;
    tmp_1[3]+=(unsigned long)ADC0->CLP3;
    tmp_1[4]+=(unsigned long)ADC0->CLP4;
    tmp_1[5]+=(unsigned long)ADC0->CLPS;
    tmp_1[6]+=(unsigned long)ADC0->CLPD;
    
    tmp_2[0]+=(unsigned long)ADC0->CLM0;
    tmp_2[1]+=(unsigned long)ADC0->CLM1;
    tmp_2[2]+=(unsigned long)ADC0->CLM2;
    tmp_2[3]+=(unsigned long)ADC0->CLM3;
    tmp_2[4]+=(unsigned long)ADC0->CLM4;
    tmp_2[5]+=(unsigned long)ADC0->CLMS;
    tmp_2[6]+=(unsigned long)ADC0->CLMD;
  }
  adc0_calib.OFS  = tmp >> TWO_POWER_N_SAMPLES;
  
  adc0_calib.CLP[0] = tmp_1[0] >> TWO_POWER_N_SAMPLES;  
  adc0_calib.CLP[1] = tmp_1[1] >> TWO_POWER_N_SAMPLES;  
  adc0_calib.CLP[2] = tmp_1[2] >> TWO_POWER_N_SAMPLES;  
  adc0_calib.CLP[3] = tmp_1[3] >> TWO_POWER_N_SAMPLES;  
  adc0_calib.CLP[4] = tmp_1[4] >> TWO_POWER_N_SAMPLES;  
  adc0_calib.CLPS   = tmp_1[5] >> TWO_POWER_N_SAMPLES;  
  adc0_calib.CLPD   = tmp_1[6] >> TWO_POWER_N_SAMPLES;    
  
  tmp = tmp_1[0]+tmp_1[1]+tmp_1[2]+tmp_1[3]+tmp_1[4]+tmp_1[5];
  adc0_calib.PG  = ((tmp>>(1+TWO_POWER_N_SAMPLES))|0x8000);
  
  adc0_calib.CLM[0] = tmp_2[0] >> TWO_POWER_N_SAMPLES;  
  adc0_calib.CLM[1] = tmp_2[1] >> TWO_POWER_N_SAMPLES;  
  adc0_calib.CLM[2] = tmp_2[2] >> TWO_POWER_N_SAMPLES;  
  adc0_calib.CLM[3] = tmp_2[3] >> TWO_POWER_N_SAMPLES;  
  adc0_calib.CLM[4] = tmp_2[4] >> TWO_POWER_N_SAMPLES;  
  adc0_calib.CLMS   = tmp_2[5] >> TWO_POWER_N_SAMPLES;  
  adc0_calib.CLMD   = tmp_2[6] >> TWO_POWER_N_SAMPLES;    
  
  tmp = tmp_2[0]+tmp_2[1]+tmp_2[2]+tmp_2[3]+tmp_2[4]+tmp_2[5];
  adc0_calib.MG  = ((tmp>>(1+TWO_POWER_N_SAMPLES))|0x8000);
  
  ADC0->CLP0 = adc0_calib.CLP[0]; 
  ADC0->CLP1 = adc0_calib.CLP[1];
  ADC0->CLP2 = adc0_calib.CLP[2];
  ADC0->CLP3 = adc0_calib.CLP[3];
  ADC0->CLP4 = adc0_calib.CLP[4];
  ADC0->CLPS = adc0_calib.CLPS;
  ADC0->CLPD = adc0_calib.CLPD;  
  ADC0->CLM0 = adc0_calib.CLM[0];
  ADC0->CLM1 = adc0_calib.CLM[1];
  ADC0->CLM2 = adc0_calib.CLM[2];
  ADC0->CLM3 = adc0_calib.CLM[3];
  ADC0->CLM4 = adc0_calib.CLM[4];
  ADC0->CLMS = adc0_calib.CLMS;
  ADC0->CLMD = adc0_calib.CLMD;
  ADC0->PG   = adc0_calib.PG; 
  ADC0->MG   = adc0_calib.MG; 
  ADC0->OFS  = adc0_calib.OFS;  

  /* End of ADC calibration */

  enable_irq(ADC0_IRQn);
  set_irq_priority(ADC0_IRQn, ISR_PRIORITY_ADC0);
  
  // Bus clock / 1 (24 MHz), 12-bit, Bus clock source
  ADC0->CFG1 = ADC_CFG1_ADIV(0) | ADC_CFG1_MODE(1) | ADC_CFG1_ADICLK(0);
  // high-speed mode, 2 extra ADC clock cycles; 6 ADCK cycles sample time
  ADC0->CFG2 = ADC_CFG2_ADHSC_MASK | ADC_CFG2_ADLSTS(3);
  ADC0->CV1  = 1;
  ADC0->CV2  = 1;
  // H/W trigger, DMA disabled
  ADC0->SC2  = ADC_SC2_ADTRG_MASK | ADC_SC2_REFSEL(0);
  // Average disabled
  ADC0->SC3  = ADC_SC3_AVGS(0);
  ADC0->SC1[0] = ADC_CHANNEL_DCBV;
  ADC0->SC1[1] = ADC_CHANNEL_DCBV | ADC_SC1_AIEN_MASK;
  return 0;
}

/*************************************************************************
 * Function Name: PDB_init
 * Parameters: none
 * Return: none
 * Description: PDB module initialization
 *************************************************************************/
void PDB_init(void)
{
  PDB0->CH[0].C1 = PDB_C1_TOS(3) | PDB_C1_EN(3) | PDB_C1_BB(2);
  PDB0->CH[0].DLY[0] = PWM_MODULO / 6;
  
  PDB0->SC = PDB_SC_LDMOD(0) | PDB_SC_PDBEN_MASK;
  PDB0->MOD = 0x7fff;
  PDB0->SC = PDB_SC_LDMOD(2) | PDB_SC_PDBEIE_MASK | (0 << PDB_SC_PRESCALER_SHIFT) | PDB_SC_TRGSEL(8) | \
            PDB_SC_PDBEN_MASK | (0 << PDB_SC_MULT_SHIFT) | PDB_SC_LDOK_MASK;

  enable_irq(PDB0_IRQn);
  set_irq_priority(PDB0_IRQn, ISR_PRIORITY_PDB0);
}

/*************************************************************************
 * Function Name: FTM0_init
 * Parameters: none
 * Return: none
 * Description: FlexTimer 0 initialization
 *************************************************************************/
void FTM0_init(void)
{
  FTM0->MODE = FTM_MODE_WPDIS_MASK | FTM_MODE_FTMEN_MASK;
  
  FTM0->CNTIN = (unsigned) -PWM_MODULO/2;
  FTM0->MOD = PWM_MODULO/2 - 1;       // 20 kHz

  FTM0->CONTROLS[0].CnSC = FTM_CnSC_ELSB_MASK;
  FTM0->CONTROLS[1].CnSC = FTM_CnSC_ELSB_MASK;
  FTM0->CONTROLS[2].CnSC = FTM_CnSC_ELSB_MASK;
  FTM0->CONTROLS[3].CnSC = FTM_CnSC_ELSB_MASK;
  FTM0->CONTROLS[4].CnSC = FTM_CnSC_ELSB_MASK;
  FTM0->CONTROLS[5].CnSC = FTM_CnSC_ELSB_MASK;

  FTM0->SYNC = FTM_SYNC_CNTMAX_MASK | FTM_SYNC_SYNCHOM_MASK;     // output mask updated on PWM synchronization (not on rising edge of clock)

  FTM0->COMBINE = FTM_COMBINE_SYNCEN2_MASK | FTM_COMBINE_DTEN2_MASK | FTM_COMBINE_COMP2_MASK | FTM_COMBINE_COMBINE2_MASK | \
                 FTM_COMBINE_SYNCEN1_MASK | FTM_COMBINE_DTEN1_MASK | FTM_COMBINE_COMP1_MASK | FTM_COMBINE_COMBINE1_MASK | \
                 FTM_COMBINE_SYNCEN0_MASK | FTM_COMBINE_DTEN0_MASK | FTM_COMBINE_COMP0_MASK | FTM_COMBINE_COMBINE0_MASK;

  // DTPS - deadtime prescaler: 0,1 = 1; 2 = 4; 3 = 16
  // DTVAL - deadtime value (0-63): deadtime period = DTPS x DTVAL
  FTM0->DEADTIME = FTM_DEADTIME_DTPS(0) | FTM_DEADTIME_DTVAL(63);

  FTM0->EXTTRIG = FTM_EXTTRIG_INITTRIGEN_MASK;

  FTM0->POL = FTM_POL_POL0_MASK | FTM_POL_POL2_MASK | FTM_POL_POL4_MASK;       // High transistors have negative polarity (MC33927/37)

  /* Following line configures:
     - enhanced PWM synchronization, FTM counter reset on SW sync
     - output SW control / polarity registers updated on PWM synchronization (not on rising edge of clock)
     - output SW control/inverting(swap)/mask registers updated from buffers on SW synchronization */  
  FTM0->SYNCONF = FTM_SYNCONF_SYNCMODE_MASK | FTM_SYNCONF_SWRSTCNT_MASK | \
                 FTM_SYNCONF_SWOC_MASK | FTM_SYNCONF_INVC_MASK | \
                 FTM_SYNCONF_SWSOC_MASK | FTM_SYNCONF_SWINVC_MASK | FTM_SYNCONF_SWOM_MASK;
  
  FTM0->OUTMASK = FTM_OUTMASK_CH0OM_MASK | FTM_OUTMASK_CH1OM_MASK | FTM_OUTMASK_CH2OM_MASK | FTM_OUTMASK_CH3OM_MASK | FTM_OUTMASK_CH4OM_MASK | FTM_OUTMASK_CH5OM_MASK;

  FTM0->CNT = 1;         // update of FTM settings
  
  FTM0->SC = FTM_SC_CLKS(1);     // no ISR, counting up, system clock, divide by 1
  
  FTM0->CONTROLS[0].CnV = (unsigned) -PWM_MODULO/4;
  FTM0->CONTROLS[1].CnV = PWM_MODULO/4;
  FTM0->CONTROLS[2].CnV = (unsigned) -PWM_MODULO/4;
  FTM0->CONTROLS[3].CnV = PWM_MODULO/4;
  FTM0->CONTROLS[4].CnV = (unsigned) -PWM_MODULO/4;
  FTM0->CONTROLS[5].CnV = PWM_MODULO/4;

  FTM0->SYNC |= FTM_SYNC_SWSYNC_MASK;
  
  FTM0->PWMLOAD |= FTM_PWMLOAD_LDOK_MASK;

  // FTM0 PWM output pins
  PORTC->PCR[1]  = PORT_PCR_MUX(4); // Drive Strength Enable
  PORTC->PCR[3]  = PORT_PCR_MUX(4); // Drive Strength Enable
  PORTC->PCR[4]  = PORT_PCR_MUX(4); // Drive Strength Enable
  PORTD->PCR[4]  = PORT_PCR_MUX(4); // Drive Strength Enable
  PORTD->PCR[5]  = PORT_PCR_MUX(4); // Drive Strength Enable
  PORTE->PCR[25] = PORT_PCR_MUX(3); // Drive Strength Enable
 
  GPIOC->PDDR |= (1 << 1) | (1 << 3) | (1 << 4);
  GPIOD->PDDR |= (1 << 4) | (1 << 5);
  GPIOE->PDDR |= 1 << 25;  
}

/*************************************************************************
 * Function Name: FTM1_init
 * Parameters: none
 * Return: none
 * Description: FlexTimer 1 initialization
 *************************************************************************/
void FTM1_init(void)
{
  FTM1->MODE = FTM_MODE_WPDIS_MASK | FTM_MODE_FTMEN_MASK;
  // Free running timer
  FTM1->MOD = 0xffff;

  // system clock, divide by 128, 1.70666us @ 75 MHz clock, 1.7777us @ 72 MHz clock
  FTM1->SC = FTM_SC_CLKS(1) | FTM_SC_PS(7);

  // Enable Output Compare interrupt, output Compare, Software Output Compare only (ELSnB:ELSnA = 0:0, output pin is not controlled by FTM)
  FTM1->CONTROLS[0].CnSC = FTM_CnSC_MSA_MASK | FTM_CnSC_CHIE_MASK;
  
  enable_irq(FTM1_IRQn);
  set_irq_priority(FTM1_IRQn, ISR_PRIORITY_FORCED_CMT);
}

/*************************************************************************
 * Function Name: FTM2_init
 * Parameters: none
 * Return: none
 * Description: FlexTimer 2 initialization
 *************************************************************************/
void FTM2_init(void)
{
  FTM2->MODE = FTM_MODE_WPDIS_MASK | FTM_MODE_FTMEN_MASK;
  // Free running timer
  FTM2->MOD = 0xffff;

  // system clock, divide by 2
  FTM2->SC = FTM_SC_CLKS(1) | FTM_SC_PS(1);

  // Enable Output Compare interrupt, output Compare, Software Output Compare only (ELSnB:ELSnA = 0:0, output pin is not controlled by FTM)
  FTM2->CONTROLS[0].CnSC = FTM_CnSC_MSA_MASK | FTM_CnSC_CHIE_MASK;
  
  enable_irq(FTM2_IRQn);
  set_irq_priority(FTM2_IRQn, ISR_PRIORITY_SLOW_TIMER);
}

/*************************************************************************
 * Function Name: SPI0_init
 * Parameters: none
 * Return: none
 * Description: SPI initialization
 *************************************************************************/
void SPI0_init(void)
{
  PORTC->PCR[2] = PORT_PCR_MUX(1);     // MC33937 RESET
  GPIOC->PDDR |= 1 << 2;
  PORTD->PCR[7] = PORT_PCR_MUX(1);     // MC33937 DRV_EN
  GPIOD->PDDR |= 1 << 7;
  
  MC33937_RESET_HIGH;
  MC33937_ENABLE_HIGH;

  SPI0->MCR = (uint32_t)0x01UL;   // halt SPI before SPI setting
  SPI0->MCR &= ~SPI_MCR_MDIS_MASK; 
  SPI0->MCR |= SPI_MCR_MSTR_MASK | SPI_MCR_PCSIS(1) | SPI_MCR_DIS_TXF_MASK | SPI_MCR_DIS_RXF_MASK;
  SPI0->TCR = (uint32_t)0x00UL;                  
  SPI0->CTAR[0] = (uint32_t)0x3A010000UL;
  SPI0->CTAR[0] |= SPI_CTAR_PCSSCK(3) | SPI_CTAR_PBR(3) | SPI_CTAR_PASC(2)| SPI_CTAR_DT(2) | SPI_CTAR_BR(3);
  SPI0->SR = (uint32_t)0x9A2A0000UL;                  
  SPI0->RSER = (uint32_t)0x00UL;                  
  SPI0->MCR &= (uint32_t)~0x01UL;      
  SPI0->SR |= SPI_SR_EOQF_MASK;
  
  SPI0->MCR &= ~SPI_MCR_HALT_MASK; // starts DSPI transfers

  PORTD->PCR[3] = PORT_PCR_MUX(2);
  PORTD->PCR[2] = PORT_PCR_MUX(2);
  PORTC->PCR[0] = PORT_PCR_MUX(7);
  PORTC->PCR[5] = PORT_PCR_MUX(2);
} 

/*************************************************************************
 * Function Name: UART1_init
 * Parameters: none
 * Return: none
 * Description: UART port configuration
 *************************************************************************/
void UART1_init(void)
{
  // Pooling mode
  UART1->C2 &= ~(UART_C2_TE_MASK|UART_C2_RE_MASK );  /* disable uart operation */

  UART1->BDH     |= ((BUS_CLOCK / (UART_BAUDRATE * 16)) >> 8) & 0x1f;
  UART1->BDL     = (BUS_CLOCK / (UART_BAUDRATE * 16)) & 0xff;
  
  UART1->C1      = 0;
  UART1->C2      = UART_C2_TE_MASK | UART_C2_RE_MASK;                           /* enable uart operation  */
  
  // Configure UART1 port pins: PTD0, PTD1 - ALT5
  PORTD->PCR[0] = PORT_PCR_MUX(5);
  PORTD->PCR[1] = PORT_PCR_MUX(5);
}

/*************************************************************************
 * Function Name: GPIO_init
 * Parameters: none
 * Return: none
 * Description: I/O pins configuration
 *************************************************************************/
void GPIO_init(void)
{
  // Brake control
  PORTE->PCR[24] = PORT_PCR_MUX(1);
  GPIOE->PDDR |= 1 << 24;
  GPIOE->PCOR = 1 << 24;  // Brake resistor off

  // LED
  PORTD->PCR[6] = PORT_PCR_MUX(1);
  GPIOD->PDDR |= 1 << 6;
  GPIOD->PSOR = 1 << 6;   // Turn off LED
  
  // Push buttons: PDB0 = SW2, PTA4 = SW1
  PORTA->PCR[4] = PORT_PCR_MUX(1);
  PORTB->PCR[0] = PORT_PCR_MUX(1);
}

/*************************************************************************
 * Function Name: SPI_Send
 * Parameters: none
 * Return: none
 * Description: SPI send data handler
 *************************************************************************/
unsigned char SPI_Send(unsigned char spi_data)
{
   SPI0->PUSHR = SPI_PUSHR_TXDATA(spi_data) | SPI_PUSHR_PCS_MASK;
   //wait till transmission completes
   while (!((SPI0->SR >> SPI_SR_TCF_SHIFT) & 1)){}; 
   SPI0->SR |= SPI_SR_TCF_MASK;  // clear TCF flag
   return SPI0->POPR; 
}

/*************************************************************************
 * Function Name: MC33927_readStatus
 * Parameters: none
 * Return: none
 * Description: Reads MC33937 driver status registers
 *************************************************************************/
void MC33927_readStatus(void)
{
  /* This command reads Status Register 0 of MC33927 and stores it in the mudtStatusRegister0 structure */
  MC33927_GetSR0(&mudtStatusRegister0);
    
  /* This command reads Status Register 1 of MC33927 and stores it in the mudtStatusRegister1 structure */    
  MC33927_GetSR1(&mudtStatusRegister1);
  
  /* This command reads Status Register 2 of MC33927 and stores it in the mudtStatusRegister2 structure */
  MC33927_GetSR2(&mudtStatusRegister2);
}

/*************************************************************************
 * Function Name: MC33927_config
 * Parameters: none
 * Return: none
 * Description: Initializes the MC33937 driver
 *************************************************************************/
void MC33927_config(void)
{
    /* This command reads Status Register 0 of MC33927 and stores it in the mudtStatusRegister0 structure */
    MC33927_GetSR0(&mudtStatusRegister0);
    
    /* This command reads Status Register 1 of MC33927 and stores it in the mudtStatusRegister1 structure */    
    MC33927_GetSR1(&mudtStatusRegister1);
    
    /* This command reads Status Register 2 of MC33927 and stores it in the mudtStatusRegister2 structure */
    MC33927_GetSR2(&mudtStatusRegister2);

    /* This command reads Status Register 3 (deadtime) of MC33927 and stores it in the mbytDeadtime variable */
    mbytDeadtime = MC33927_GetSR3();
    
    /***** Mode setup *****/
    mudtModeCommands.B.Lock = 0; /* 0: allows further writing; 1: locks the configuration until reset */
    mudtModeCommands.B.FullOnMode = 0; /* 0: BLDC, PMSM, ACIM; 1: SR motors */    
    mudtModeCommands.B.DesatFaultMode = 0; /* 0: when desaturation or phase error
                                            on any MOSFET turns off all MOSFETs; 1: desat faults ignored */
    
    /* This command controls the Desaturation Fault Mode, Full On Mode and Lock Mode bits */
    MC33927_ModeCommand(&mudtModeCommands);
    
    /***** Interrupts setup *****/    
    mudtMaskInterrupts.B.TLim = 0; /* overtemperature interrupt 0: disables; 1: enables */
    mudtMaskInterrupts.B.Desat = 0; /* dasaturation interrupt 0: disables; 1: enables */
    mudtMaskInterrupts.B.LowVLS = 0; /* VLS undervoltage interrupt 0: disables; 1: enables */
    mudtMaskInterrupts.B.Overcurrent = 0; /* overcurrent interrupt 0: disables; 1: enables */
    mudtMaskInterrupts.B.PhaseError = 0; /* phase error interrupt 0: disables; 1: enables */
    mudtMaskInterrupts.B.FramingError = 0; /* SPI framing error interrupt 0: disables; 1: enables */
    mudtMaskInterrupts.B.WriteError = 0; /* write after lock is on interrupt 0: disables; 1: enables */
    mudtMaskInterrupts.B.ResetEvent = 0; /* driver set or disabled interrupt 0: disables; 1: enables */
    
    /* This command controls interrupt mask registers */
    MC33927_MaskInterrupts(&mudtMaskInterrupts);

    /***** Flags clearing *****/    
    mudtClearFlags.B.TLim = 1; /* overtemperature flag 0: untouched; 1: clears */
    mudtClearFlags.B.Desat = 1; /* dasaturation flag 0: untouched; 1: clears */
    mudtClearFlags.B.LowVLS = 1; /* VLS undervoltage flag 0: untouched; 1: clears */
    mudtClearFlags.B.Overcurrent = 1; /* overcurrent flag 0: untouched; 1: clears */
    mudtClearFlags.B.PhaseError = 1; /* phase error flag 0: untouched; 1: clears */
    mudtClearFlags.B.FramingError = 1; /* SPI framing error flag 0: untouched; 1: clears */
    mudtClearFlags.B.WriteError = 1; /* write after lock is on flag 0: untouched; 1: clears */
    mudtClearFlags.B.ResetEvent = 1; /* driver set upon exiting /RST flag 0: untouched; 1: clears */
    
    for (int ind=0; ind <10000;ind++) asm("nop"); // Delay has to be inserted, otherwise 3PPA driver didn't eat the command
    /* This command clears the flag registers */
    MC33927_ClearFlags(&mudtClearFlags);
    for (int ind=0; ind <10000;ind++) asm("nop"); // Delay has to be inserted, otherwise 3PPA driver didn't eat the command
    /* This command sets the zero deadtime */
    MC33927_ZeroDeadtime();
    
    /* Reading the status registers to check if it was set correctly */
    MC33927_GetSR0(&mudtStatusRegister0); /* SR0 */
    MC33927_GetSR1(&mudtStatusRegister1); /* SR1 */
    MC33927_GetSR2(&mudtStatusRegister2); /* SR2 */
    mbytDeadtime = MC33927_GetSR3(); /* SR3, deadtime value */
}

/*************************************************************************
 * Function Name: FTM0_SetDutyCycle
 * Parameters: duty cycle value
 * Return: none
 * Description: Set PWM dutycycle
 *************************************************************************/
void FTM0_SetDutyCycle(int16 inpDuty)
{
int FirstEdge, SecondEdge, duty;

  duty = MLIB_Mul_F16(inpDuty, PWM_MODULO/4);

  FirstEdge = -PWM_MODULO/4 - duty;
  if (FirstEdge < (-PWM_MODULO/2)) FirstEdge = -PWM_MODULO/2;
  
  SecondEdge = PWM_MODULO/4 + duty;
  if (SecondEdge > (PWM_MODULO/2)) SecondEdge = PWM_MODULO/2;

  FTM0->CONTROLS[0].CnV = FirstEdge;
  FTM0->CONTROLS[1].CnV = SecondEdge;
  FTM0->CONTROLS[2].CnV = FirstEdge;
  FTM0->CONTROLS[3].CnV = SecondEdge;
  FTM0->CONTROLS[4].CnV = FirstEdge;
  FTM0->CONTROLS[5].CnV = SecondEdge;
  FTM0->PWMLOAD |= FTM_PWMLOAD_LDOK_MASK;
}

/*************************************************************************
 * Function Name: FTM0_SetPwmOutput
 * Parameters: sector number
 * Return: none
 * Description: set PWM output configuration based on selected sector
 *************************************************************************/
void FTM0_SetPwmOutput(int16 sector)
{ 
  FTM0->INVCTRL = bldcCommutationTableComp[sector].swap;
  FTM0->OUTMASK = bldcCommutationTableComp[sector].mask;
  
  FTM0->SYNC |= FTM_SYNC_SWSYNC_MASK;
}

/*************************************************************************
 * Function Name: MCU_init
 * Parameters: none
 * Return: none
 * Description: initialization of peripheral modules after reset 
 *************************************************************************/
void MCU_init(void)
{
  Clock_init();  
  ADC0_init();
  PDB_init();
  FTM0_init();
  FTM1_init();
  FTM2_init();
  SPI0_init();
  UART1_init();
  GPIO_init();
  MC33927_config();
}
             

