
/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file Blinky_orange.c
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun. 25, 2013
*
* @brief providing framework of demo cases for MCU. 
*
*******************************************************************************/

#include "common.h"
#include "mcg.h"
#include "uart.h"
#include "kv10_config.h"
/******************************************************************************
* Global variables
******************************************************************************/

/******************************************************************************
* Constants and macros
******************************************************************************/

/******************************************************************************
* Local types
******************************************************************************/

/******************************************************************************
* Local function prototypes
******************************************************************************/

/******************************************************************************
* Local variables
******************************************************************************/

/******************************************************************************
* Local functions
******************************************************************************/
int main (void);
void LPTMR_Init(int count, int clock_source);
void LPTMR_Isr(void);
/******************************************************************************
* Global functions
******************************************************************************/

/********************************************************************/
int main (void)
{
    uint8_t         u8Ch; 
  	printf("\nRunning the Blinky_orange project.\n");
    LED1_Init();
    LED3_Init();
    LED5_Init();
    
    LED1_On();
    LED3_On();
    LED5_On();
    /*select LPO as LPTMR clock source */
    LPTMR_Init(1000,1);
    printf("Please enter any character which will echo...\n");
    
    /* echo chars received from terminal */
	while(1)
	{
        u8Ch = UART_GetChar(TERM_PORT);
        UART_PutChar(TERM_PORT, u8Ch);
	} 

}
void LPTMR_Isr(void)
{
    /* write 1 to TCF to clear the LPT timer compare flag */
    LPTMR0->CSR |=  LPTMR_CSR_TCF_MASK;   
    LED1_Toggle();
    LED3_Toggle();
    LED5_Toggle();
}
/*******************************************************************************
*
*   PROCEDURE NAME:
*       lptmr_init -
*
*******************************************************************************/
void LPTMR_Init(int count, int clock_source)
{
    SIM->SCGC5 |= SIM_SCGC5_LPTMR_MASK;
    // disable LPTMR
    LPTMR0->CSR &= ~LPTMR_CSR_TEN_MASK; 
    // select LPO for RTC and LPTMR
    LPTMR0->PSR = ( LPTMR_PSR_PRESCALE(0) // 0000 is div 2
                 | LPTMR_PSR_PBYP_MASK  // LPO feeds directly to LPT
                 | LPTMR_PSR_PCS(clock_source)) ; // use the choice of clock
              
    LPTMR0->CMR = LPTMR_CMR_COMPARE(count);  //Set compare value
	// clear flag
	LPTMR0->CSR |=  LPTMR_CSR_TCF_MASK; 
	
    LPTMR0->CSR =(  LPTMR_CSR_TCF_MASK   // Clear any pending interrupt
                 | LPTMR_CSR_TIE_MASK   // LPT interrupt enabled
                 );
	
    LPTMR0->CSR |= LPTMR_CSR_TEN_MASK;   //Turn on LPT and start counting
    
    NVIC_EnableIRQ(LPTMR0_IRQn);

}